@@ -15,6 +15,7 @@
 import contextlib
 import errno
 import hashlib
+import importlib
 import logging
 import math
 import os
@@ -37,7 +38,7 @@ import glance_store.location
 
 try:
     from cinderclient import exceptions as cinder_exception
-    from cinderclient.v2 import client as cinderclient
+    from cinderclient.v3 import client as cinderclient
     from os_brick.initiator import connector
     from oslo_privsep import priv_context
 except ImportError:
@@ -304,84 +305,52 @@ Possible values:
 Related options:
     * None
 
+NOTE: You cannot use an encrypted volume_type associated with an NFS backend.
+An encrypted volume stored on an NFS backend will raise an exception whenever
+glance_store tries to write or access image data stored in that volume.
+Consult your Cinder administrator to determine an appropriate volume_type.
+
+"""),
+    cfg.BoolOpt('cinder_enforce_multipath',
+                default=False,
+                help="""
+If this is set to True, attachment of volumes for image transfer will
+be aborted when multipathd is not running. Otherwise, it will fallback
+to single path.
+
+Possible values:
+    * True or False
+
+Related options:
+    * cinder_use_multipath
+
+"""),
+    cfg.BoolOpt('cinder_use_multipath',
+                default=False,
+                help="""
+Flag to identify mutipath is supported or not in the deployment.
+
+Set it to False if multipath is not supported.
+
+Possible values:
+    * True or False
+
+Related options:
+    * cinder_enforce_multipath
+
+"""),
+    cfg.StrOpt('cinder_mount_point_base',
+               default='/var/lib/glance/mnt',
+               help="""
+Directory where the NFS volume is mounted on the glance node.
+
+Possible values:
+
+* A string representing absolute path of mount point.
 """),
 ]
 
 
-def get_root_helper(backend=None):
-    if backend:
-        rootwrap = getattr(CONF, backend).rootwrap_config
-    else:
-        rootwrap = CONF.glance_store.rootwrap_config
-
-    return 'sudo glance-rootwrap %s' % rootwrap
-
-
-def is_user_overriden(conf, backend=None):
-    if backend:
-        store_conf = getattr(conf, backend)
-    else:
-        store_conf = conf.glance_store
-
-    return all([store_conf.get('cinder_store_' + key)
-                for key in ['user_name', 'password',
-                            'project_name', 'auth_address']])
-
-
-def get_cinderclient(conf, context=None, backend=None):
-    if backend:
-        glance_store = getattr(conf, backend)
-    else:
-        glance_store = conf.glance_store
-
-    user_overriden = is_user_overriden(conf, backend=backend)
-    if user_overriden:
-        username = glance_store.cinder_store_user_name
-        password = glance_store.cinder_store_password
-        project = glance_store.cinder_store_project_name
-        url = glance_store.cinder_store_auth_address
-    else:
-        username = context.user
-        password = context.auth_token
-        project = context.tenant
-
-        if glance_store.cinder_endpoint_template:
-            url = glance_store.cinder_endpoint_template % context.to_dict()
-        else:
-            info = glance_store.cinder_catalog_info
-            service_type, service_name, interface = info.split(':')
-            try:
-                catalog = keystone_sc.ServiceCatalogV2(context.service_catalog)
-                url = catalog.url_for(
-                    region_name=glance_store.cinder_os_region_name,
-                    service_type=service_type,
-                    service_name=service_name,
-                    interface=interface)
-            except keystone_exc.EndpointNotFound:
-                reason = _("Failed to find Cinder from a service catalog.")
-                raise exceptions.BadStoreConfiguration(store_name="cinder",
-                                                       reason=reason)
-
-    c = cinderclient.Client(username,
-                            password,
-                            project,
-                            auth_url=url,
-                            region_name=glance_store.cinder_os_region_name,
-                            insecure=glance_store.cinder_api_insecure,
-                            retries=glance_store.cinder_http_retries,
-                            cacert=glance_store.cinder_ca_certificates_file)
-
-    LOG.debug('Cinderclient connection created for user %(user)s using URL: '
-              '%(url)s.', {'user': username, 'url': url})
-
-    # noauth extracts user_id:project_id from auth_token
-    if not user_overriden:
-        c.client.auth_token = context.auth_token or '%s:%s' % (username,
-                                                               project)
-    c.client.management_url = url
-    return c
-
-
 class StoreLocation(glance_store.location.StoreLocation):
 
     """Class describing a Cinder URI."""
@@ -391,13 +360,16 @@ class StoreLocation(glance_store.location.StoreLocation):
         self.volume_id = self.specs.get('volume_id')
 
     def get_uri(self):
+        if self.backend_group:
+            return "cinder://%s/%s" % (self.backend_group,
+                                       self.volume_id)
         return "cinder://%s" % self.volume_id
 
     def parse_uri(self, uri):
         self.validate_schemas(uri, valid_schemas=('cinder://',))
 
         self.scheme = 'cinder'
-        self.volume_id = uri[9:]
+        self.volume_id = uri.split('/')[-1]
 
         if not utils.is_uuid_like(self.volume_id):
             reason = _("URI contains invalid volume ID")
@@ -405,24 +377,6 @@ class StoreLocation(glance_store.location.StoreLocation):
             raise exceptions.BadStoreUri(message=reason)
 
 
-@contextlib.contextmanager
-def temporary_chown(path, backend=None):
-    owner_uid = os.getuid()
-    orig_uid = os.stat(path).st_uid
-
-    if orig_uid != owner_uid:
-        processutils.execute('chown', owner_uid, path,
-                             run_as_root=True,
-                             root_helper=get_root_helper(backend=backend))
-    try:
-        yield
-    finally:
-        if orig_uid != owner_uid:
-            processutils.execute('chown', orig_uid, path,
-                                 run_as_root=True,
-                                 root_helper=get_root_helper(backend=backend))
-
-
 class Store(glance_store.driver.Store):
 
     """Cinder backend store adapter."""
@@ -435,13 +389,176 @@ class Store(glance_store.driver.Store):
 
     def __init__(self, *args, **kargs):
         super(Store, self).__init__(*args, **kargs)
+        # We are importing it here to let the config options load
+        # before we use them in the fs_mount file
+        self.mount = importlib.import_module('glance_store.common.fs_mount')
+        self._set_url_prefix()
+        if self.backend_group:
+            self.store_conf = getattr(self.conf, self.backend_group)
+        else:
+            self.store_conf = self.conf.glance_store
+
+    def _set_url_prefix(self):
+        self._url_prefix = "cinder://"
+        if self.backend_group:
+            self._url_prefix = "cinder://%s" % self.backend_group
+
+    def configure_add(self):
+        """
+        Configure the Store to use the stored configuration options
+        Any store that needs special configuration should implement
+        this method. If the store was not able to successfully configure
+        itself, it should raise `exceptions.BadStoreConfiguration`
+        :raises: `exceptions.BadStoreConfiguration` if multiple stores are
+                 defined and particular store wasn't able to configure
+                 successfully
+        :raises: `exceptions.BackendException` if single store is defined and
+                 it wasn't able to configure successfully
+        """
+        if self.backend_group:
+            cinder_volume_type = self.store_conf.cinder_volume_type
+            if cinder_volume_type:
+                # NOTE: `cinder_volume_type` is configured, check
+                # configured volume_type is available in cinder or not
+                cinder_client = self.get_cinderclient()
+                try:
+                    # We don't even need the volume type object, as long
+                    # as this returns clean, we know the name is good.
+                    cinder_client.volume_types.find(name=cinder_volume_type)
+                    # No need to worry NoUniqueMatch as volume type name is
+                    # unique
+                except cinder_exception.NotFound:
+                    reason = _("Invalid `cinder_volume_type %s`"
+                               % cinder_volume_type)
+                    if len(self.conf.enabled_backends) > 1:
+                        LOG.error(reason)
+                        raise exceptions.BadStoreConfiguration(
+                            store_name=self.backend_group, reason=reason)
+                    else:
+                        LOG.critical(reason)
+                        raise exceptions.BackendException(reason)
+
+    def is_image_associated_with_store(self, context, volume_id):
+        """
+        Updates legacy images URL to respective stores.
+        This method checks the volume type of the volume associated with the
+        image against the configured stores. It returns true if the
+        cinder_volume_type configured in the store matches with the volume
+        type of the image-volume. When cinder_volume_type is not configured
+        then the it checks it against default_volume_type set in cinder.
+        If above both conditions doesn't meet, it returns false.
+        """
+        try:
+            cinder_client = self.get_cinderclient(context=context,
+                                                  legacy_update=True)
+            cinder_volume_type = self.store_conf.cinder_volume_type
+            volume = cinder_client.volumes.get(volume_id)
+            if cinder_volume_type and volume.volume_type == cinder_volume_type:
+                return True
+            elif not cinder_volume_type:
+                default_type = cinder_client.volume_types.default()
+                if volume.volume_type == default_type.name:
+                    return True
+        except Exception:
+            # Glance calls this method to update legacy images URL
+            # If an exception occours due to image/volume is non-existent or
+            # any other reason, we return False (i.e. the image location URL
+            # won't be updated) and it is glance's responsibility to handle
+            # the case when the image failed to update
+            pass
+
+        return False
+
+    def get_root_helper(self):
+        rootwrap = self.store_conf.rootwrap_config
+        return 'sudo glance-rootwrap %s' % rootwrap
+
+    def is_user_overriden(self):
+        return all([self.store_conf.get('cinder_store_' + key)
+                    for key in ['user_name', 'password',
+                                'project_name', 'auth_address']])
+
+    def get_cinderclient(self, context=None, legacy_update=False):
+        # NOTE: For legacy image update from single store to multiple
+        # stores we need to use admin context rather than user provided
+        # credentials
+        if legacy_update:
+            user_overriden = False
+            context = context.elevated()
+        else:
+            user_overriden = self.is_user_overriden()
+
+        if user_overriden:
+            username = self.store_conf.cinder_store_user_name
+            password = self.store_conf.cinder_store_password
+            project = self.store_conf.cinder_store_project_name
+            url = self.store_conf.cinder_store_auth_address
+        else:
+            username = context.user
+            password = context.auth_token
+            project = context.tenant
+
+            if self.store_conf.cinder_endpoint_template:
+                template = self.store_conf.cinder_endpoint_template
+                url = template % context.to_dict()
+            else:
+                info = self.store_conf.cinder_catalog_info
+                service_type, service_name, interface = info.split(':')
+                try:
+                    catalog = keystone_sc.ServiceCatalogV2(
+                        context.service_catalog)
+                    url = catalog.url_for(
+                        region_name=self.store_conf.cinder_os_region_name,
+                        service_type=service_type,
+                        service_name=service_name,
+                        interface=interface)
+                except keystone_exc.EndpointNotFound:
+                    reason = _("Failed to find Cinder from a service catalog.")
+                    raise exceptions.BadStoreConfiguration(store_name="cinder",
+                                                           reason=reason)
+
+        c = cinderclient.Client(
+            username, password, project, auth_url=url,
+            region_name=self.store_conf.cinder_os_region_name,
+            insecure=self.store_conf.cinder_api_insecure,
+            retries=self.store_conf.cinder_http_retries,
+            cacert=self.store_conf.cinder_ca_certificates_file)
+
+        LOG.debug(
+            'Cinderclient connection created for user %(user)s using URL: '
+            '%(url)s.', {'user': username, 'url': url})
+
+        # noauth extracts user_id:project_id from auth_token
+        if not user_overriden:
+            c.client.auth_token = context.auth_token or '%s:%s' % (username,
+                                                                   project)
+        c.client.management_url = url
+        return c
+
+    @contextlib.contextmanager
+    def temporary_chown(self, path):
+        owner_uid = os.getuid()
+        orig_uid = os.stat(path).st_uid
+
+        if orig_uid != owner_uid:
+            processutils.execute(
+                'chown', owner_uid, path,
+                run_as_root=True,
+                root_helper=self.get_root_helper())
+        try:
+            yield
+        finally:
+            if orig_uid != owner_uid:
+                processutils.execute(
+                    'chown', orig_uid, path,
+                    run_as_root=True,
+                    root_helper=self.get_root_helper())
 
     def get_schemes(self):
         return ('cinder',)
 
     def _check_context(self, context, require_tenant=False):
-        user_overriden = is_user_overriden(self.conf,
-                                           backend=self.backend_group)
+        user_overriden = self.is_user_overriden()
         if user_overriden and not require_tenant:
             return
         if context is None:
@@ -455,12 +572,7 @@ class Store(glance_store.driver.Store):
 
     def _wait_volume_status(self, volume, status_transition, status_expected):
         max_recheck_wait = 15
-        if self.backend_group:
-            timeout = getattr(
-                self.conf, self.backend_group).cinder_state_transition_timeout
-        else:
-            timeout = self.conf.glance_store.cinder_state_transition_timeout
-
+        timeout = self.store_conf.cinder_state_transition_timeout
         volume = volume.manager.get(volume.id)
         tries = 0
         elapsed = 0
@@ -486,15 +598,35 @@ class Store(glance_store.driver.Store):
             raise exceptions.BackendException(msg)
         return volume
 
+    def get_hash_str(self, base_str):
+        """Returns string that represents SHA256 hash of base_str (in hex format).
+
+        If base_str is a Unicode string, encode it to UTF-8.
+        """
+        if isinstance(base_str, str):
+            base_str = base_str.encode('utf-8')
+        return hashlib.sha256(base_str).hexdigest()
+
+    def _get_mount_path(self, share, mount_point_base):
+        """Returns the mount path prefix using the mount point base and share.
+
+        :returns: The mount path prefix.
+        """
+        return os.path.join(mount_point_base, self.get_hash_str(share))
+
     @contextlib.contextmanager
     def _open_cinder_volume(self, client, volume, mode):
         attach_mode = 'rw' if mode == 'wb' else 'ro'
         device = None
-        root_helper = get_root_helper(backend=self.backend_group)
+        root_helper = self.get_root_helper()
         priv_context.init(root_helper=shlex.split(root_helper))
         host = socket.gethostname()
-        properties = connector.get_connector_properties(root_helper, host,
-                                                        False, False)
+        use_multipath = self.store_conf.cinder_use_multipath
+        enforce_multipath = self.store_conf.cinder_enforce_multipath
+        mount_point_base = self.store_conf.cinder_mount_point_base
+
+        properties = connector.get_connector_properties(
+            root_helper, host, use_multipath, enforce_multipath)
 
         try:
             volume.reserve(volume)
@@ -509,16 +641,41 @@ class Store(glance_store.driver.Store):
             conn = connector.InitiatorConnector.factory(
                 connection_info['driver_volume_type'], root_helper,
                 conn=connection_info)
-            device = conn.connect_volume(connection_info['data'])
+            if connection_info['driver_volume_type'] == 'nfs':
+                if volume.encrypted:
+                    volume.unreserve(volume)
+                    volume.delete()
+                    msg = (_('Encrypted volume creation for cinder nfs is not '
+                             'supported from glance_store. Failed to create '
+                             'volume %(volume_id)s')
+                           % {'volume_id': volume.id})
+                    LOG.error(msg)
+                    raise exceptions.BackendException(msg)
+
+                @utils.synchronized(connection_info['data']['export'])
+                def connect_volume_nfs():
+                    data = connection_info['data']
+                    export = data['export']
+                    vol_name = data['name']
+                    mountpoint = self._get_mount_path(
+                        export,
+                        os.path.join(mount_point_base, 'nfs'))
+                    options = data['options']
+                    self.mount.mount(
+                        'nfs', export, vol_name, mountpoint, host,
+                        root_helper, options)
+                    return {'path': os.path.join(mountpoint, vol_name)}
+                device = connect_volume_nfs()
+            else:
+                device = conn.connect_volume(connection_info['data'])
             volume.attach(None, 'glance_store', attach_mode, host_name=host)
             volume = self._wait_volume_status(volume, 'attaching', 'in-use')
             if (connection_info['driver_volume_type'] == 'rbd' and
                not conn.do_local_attach):
                 yield device['path']
             else:
-                with temporary_chown(device['path'],
-                                     backend=self.backend_group), \
-                        open(device['path'], mode) as f:
+                with self.temporary_chown(
+                        device['path']), open(device['path'], mode) as f:
                     yield f
         except Exception:
             LOG.exception(_LE('Exception while accessing to cinder volume '
@@ -532,7 +689,15 @@ class Store(glance_store.driver.Store):
 
             if device:
                 try:
-                    conn.disconnect_volume(connection_info['data'], device)
+                    if connection_info['driver_volume_type'] == 'nfs':
+                        @utils.synchronized(connection_info['data']['export'])
+                        def disconnect_volume_nfs():
+                            path, vol_name = device['path'].rsplit('/', 1)
+                            self.mount.umount(vol_name, path, host,
+                                              root_helper)
+                        disconnect_volume_nfs()
+                    else:
+                        conn.disconnect_volume(connection_info['data'], device)
                 except Exception:
                     LOG.exception(_LE('Failed to disconnect volume '
                                       '%(volume_id)s.'),
@@ -595,8 +760,7 @@ class Store(glance_store.driver.Store):
         loc = location.store_location
         self._check_context(context)
         try:
-            client = get_cinderclient(self.conf, context,
-                                      backend=self.backend_group)
+            client = self.get_cinderclient(context)
             volume = client.volumes.get(loc.volume_id)
             size = int(volume.metadata.get('image_size',
                                            volume.size * units.Gi))
@@ -630,9 +794,7 @@ class Store(glance_store.driver.Store):
 
         try:
             self._check_context(context)
-            volume = get_cinderclient(
-                self.conf, context,
-                backend=self.backend_group).volumes.get(loc.volume_id)
+            volume = self.get_cinderclient(context).volumes.get(loc.volume_id)
             return int(volume.metadata.get('image_size',
                                            volume.size * units.Gi))
         except cinder_exception.NotFound:
@@ -666,8 +828,7 @@ class Store(glance_store.driver.Store):
         """
 
         self._check_context(context, require_tenant=True)
-        client = get_cinderclient(self.conf, context,
-                                  backend=self.backend_group)
+        client = self.get_cinderclient(context)
         os_hash_value = hashlib.new(str(hashing_algo))
         checksum = hashlib.md5()
         bytes_written = 0
@@ -680,11 +841,7 @@ class Store(glance_store.driver.Store):
                     'image_size': str(image_size),
                     'image_owner': owner}
 
-        if self.backend_group:
-            volume_type = getattr(self.conf,
-                                  self.backend_group).cinder_volume_type
-        else:
-            volume_type = self.conf.glance_store.cinder_volume_type
+        volume_type = self.store_conf.cinder_volume_type
 
         LOG.debug('Creating a new volume: image_size=%d size_gb=%d type=%s',
                   image_size, size_gb, volume_type or 'None')
@@ -768,10 +925,13 @@ class Store(glance_store.driver.Store):
                    'checksum_hex': checksum_hex})
 
         image_metadata = {}
+        location_url = 'cinder://%s' % volume.id
         if self.backend_group:
-            image_metadata['backend'] = u"%s" % self.backend_group
+            image_metadata['store'] = u"%s" % self.backend_group
+            location_url = 'cinder://%s/%s' % (self.backend_group,
+                                               volume.id)
 
-        return ('cinder://%s' % volume.id,
+        return (location_url,
                 bytes_written,
                 checksum_hex,
                 hash_hex,
@@ -792,9 +952,7 @@ class Store(glance_store.driver.Store):
         loc = location.store_location
         self._check_context(context)
         try:
-            volume = get_cinderclient(
-                self.conf, context,
-                backend=self.backend_group).volumes.get(loc.volume_id)
+            volume = self.get_cinderclient(context).volumes.get(loc.volume_id)
             volume.delete()
         except cinder_exception.NotFound:
             raise exceptions.NotFound(image=loc.volume_id)
