@@ -1,4 +1,5 @@
 # Copyright 2010-2011 Josh Durgin
+# Copyright 2020 Red Hat, Inc.
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -15,8 +16,6 @@
 
 """Storage backend for RBD
    (RADOS (Reliable Autonomic Distributed Object Store) Block Device)"""
-from __future__ import absolute_import
-from __future__ import with_statement
 
 import contextlib
 import hashlib
@@ -24,6 +23,7 @@ import logging
 import math
 
 from oslo_config import cfg
+from oslo_utils import encodeutils
 from oslo_utils import units
 from six.moves import urllib
 
@@ -31,7 +31,7 @@ from glance_store import capabilities
 from glance_store.common import utils
 from glance_store import driver
 from glance_store import exceptions
-from glance_store.i18n import _, _LE, _LI
+from glance_store.i18n import _, _LE, _LI, _LW
 from glance_store import location
 
 try:
@@ -42,7 +42,6 @@ except ImportError:
     rbd = None
 
 DEFAULT_POOL = 'images'
-DEFAULT_CONFFILE = '/etc/ceph/ceph.conf'
 DEFAULT_USER = None    # let librados decide based on the Ceph conf file
 DEFAULT_CHUNKSIZE = 8  # in MiB
 DEFAULT_SNAPNAME = 'snap'
@@ -109,16 +108,23 @@ Related options:
     * rbd_store_ceph_conf
 
 """),
-    cfg.StrOpt('rbd_store_ceph_conf', default=DEFAULT_CONFFILE,
+    cfg.StrOpt('rbd_store_ceph_conf', default='',
                help="""
 Ceph configuration file path.
 
-This configuration option takes in the path to the Ceph configuration
+This configuration option specifies the path to the Ceph configuration
 file to be used. If the value for this option is not set by the user
-or is set to None, librados will locate the default configuration file
-which is located at /etc/ceph/ceph.conf. If using Cephx
-authentication, this file should include a reference to the right
-keyring in a client.<USER> section
+or is set to the empty string, librados will read the standard ceph.conf
+file by searching the default Ceph configuration file locations in
+sequential order.  See the Ceph documentation for details.
+
+NOTE: If using Cephx authentication, this file should include a reference
+to the right keyring in a client.<USER> section
+
+NOTE 2: If you leave this option empty (the default), the actual Ceph
+configuration file used may change depending on what version of librados
+is being used.  If it is important for you to know exactly which configuration
+file is in effect, you may specify that file here using this option.
 
 Possible Values:
     * A valid path to a configuration file
@@ -144,6 +150,26 @@ Possible Values:
 Related options:
     * None
 
+"""),
+    cfg.BoolOpt('rbd_thin_provisioning',
+                default=False,
+                help="""
+Enable or not thin provisioning in this backend.
+
+This configuration option enable the feature of not really write null byte
+sequences on the RBD backend, the holes who can appear will automatically
+be interpreted by Ceph as null bytes, and do not really consume your storage.
+Enabling this feature will also speed up image upload and save network trafic
+in addition to save space in the backend, as null bytes sequences are not
+sent over the network.
+
+Possible Values:
+    * True
+    * False
+
+Related options:
+    * None
+
 """),
 ]
 
@@ -241,7 +267,7 @@ class ImageIterator(object):
                             data = image.read(size - bytes_left, length)
                             bytes_left -= len(data)
                             yield data
-                        raise StopIteration()
+                        return
         except rbd.ImageNotFound:
             raise exceptions.NotFound(
                 _('RBD image %s does not exist') % self.name)
@@ -264,10 +290,16 @@ class Store(driver.Store):
 
         try:
             client.connect(timeout=self.connect_timeout)
-        except rados.Error:
-            msg = _LE("Error connecting to ceph cluster.")
-            LOG.exception(msg)
-            raise exceptions.BackendException()
+        except (rados.Error, rados.ObjectNotFound) as e:
+            if self.backend_group and len(self.conf.enabled_backends) > 1:
+                reason = _("Error in store configuration: %s") % e
+                LOG.debug(reason)
+                raise exceptions.BadStoreConfiguration(
+                    store_name=self.backend_group, reason=reason)
+            else:
+                msg = _LE("Error connecting to ceph cluster.")
+                LOG.exception(msg)
+                raise exceptions.BackendException()
         try:
             yield client
         finally:
@@ -290,13 +322,19 @@ class Store(driver.Store):
                                     self.backend_group).rbd_store_ceph_conf
                 connect_timeout = getattr(
                     self.conf, self.backend_group).rados_connect_timeout
+                thin_provisioning = getattr(self.conf,
+                                            self.backend_group).\
+                    rbd_thin_provisioning
             else:
                 chunk = self.conf.glance_store.rbd_store_chunk_size
                 pool = self.conf.glance_store.rbd_store_pool
                 user = self.conf.glance_store.rbd_store_user
                 conf_file = self.conf.glance_store.rbd_store_ceph_conf
                 connect_timeout = self.conf.glance_store.rados_connect_timeout
+                thin_provisioning = \
+                    self.conf.glance_store.rbd_thin_provisioning
 
+            self.thin_provisioning = thin_provisioning
             self.chunk_size = chunk * units.Mi
             self.READ_CHUNKSIZE = self.chunk_size
             self.WRITE_CHUNKSIZE = self.READ_CHUNKSIZE
@@ -312,6 +350,25 @@ class Store(driver.Store):
             LOG.error(reason)
             raise exceptions.BadStoreConfiguration(store_name='rbd',
                                                    reason=reason)
+        if self.backend_group:
+            self._set_url_prefix()
+        self.size = 0
+        self.resize_amount = self.WRITE_CHUNKSIZE
+
+    def _set_url_prefix(self):
+        fsid = None
+        with self.get_connection(conffile=self.conf_file,
+                                 rados_id=self.user) as conn:
+            if hasattr(conn, 'get_fsid'):
+                fsid = encodeutils.safe_decode(conn.get_fsid())
+
+        if fsid and self.pool:
+            # ensure nothing contains / or any other url-unsafe character
+            safe_fsid = urllib.parse.quote(fsid, '')
+            safe_pool = urllib.parse.quote(self.pool, '')
+            self._url_prefix = "rbd://%s/%s/" % (safe_fsid, safe_pool)
+        else:
+            self._url_prefix = "rbd://"
 
     @capabilities.check
     def get(self, location, offset=0, chunk_size=None, context=None):
@@ -406,26 +463,26 @@ class Store(driver.Store):
                                        {'snap_exc': exc})
                                 LOG.debug(msg)
                             except rbd.ImageBusy as exc:
-                                log_msg = (_LE("Snap Operating Exception "
+                                log_msg = (_LW("Snap Operating Exception "
                                                "%(snap_exc)s "
                                                "Snapshot is in use.") %
                                            {'snap_exc': exc})
-                                LOG.error(log_msg)
+                                LOG.warning(log_msg)
                                 raise exceptions.InUseByStore()
 
                     # Then delete image.
                     rbd.RBD().remove(ioctx, image_name)
                 except rbd.ImageHasSnapshots:
-                    log_msg = (_LE("Remove image %(img_name)s failed. "
+                    log_msg = (_LW("Remove image %(img_name)s failed. "
                                    "It has snapshot(s) left.") %
                                {'img_name': image_name})
-                    LOG.error(log_msg)
+                    LOG.warning(log_msg)
                     raise exceptions.HasSnapshot()
                 except rbd.ImageBusy:
-                    log_msg = (_LE("Remove image %(img_name)s failed. "
+                    log_msg = (_LW("Remove image %(img_name)s failed. "
                                    "It is in use.") %
                                {'img_name': image_name})
-                    LOG.error(log_msg)
+                    LOG.warning(log_msg)
                     raise exceptions.InUseByStore()
                 except rbd.ImageNotFound:
                     msg = _("RBD image %s does not exist") % image_name
@@ -440,6 +497,18 @@ class Store(driver.Store):
             # Such exception is not dangerous for us so it will be just logged
             LOG.debug("Snapshot %s is unprotected already" % snap_name)
 
+    def _resize_on_write(self, image, image_size, bytes_written, chunk_length):
+        """Handle the rbd resize when needed."""
+        if image_size != 0 or self.size >= bytes_written + chunk_length:
+            return self.size
+        new_size = self.size + self.resize_amount
+        LOG.debug("resizing image to %s KiB" % (new_size / units.Ki))
+        image.resize(new_size)
+        # Note(jokke): We double how much we grow the image each time
+        # up to 8gigs to avoid resizing for each write on bigger images
+        self.resize_amount = min(self.resize_amount * 2, 8 * units.Gi)
+        return new_size
+
     @driver.back_compat_add
     @capabilities.check
     def add(self, image_id, image_file, image_size, hashing_algo, context=None,
@@ -469,15 +538,26 @@ class Store(driver.Store):
                                  rados_id=self.user) as conn:
             fsid = None
             if hasattr(conn, 'get_fsid'):
-                fsid = conn.get_fsid()
+                # Librados's get_fsid is represented as binary
+                # in py3 instead of str as it is in py2.
+                # This is causing problems with ceph.
+                # Decode binary to str fixes these issues.
+                # Fix with encodeutils.safe_decode CAN BE REMOVED
+                # after librados's fix will be stable.
+                #
+                # More informations:
+                # https://bugs.launchpad.net/glance-store/+bug/1816721
+                # https://bugs.launchpad.net/cinder/+bug/1816468
+                # https://tracker.ceph.com/issues/38381
+                fsid = encodeutils.safe_decode(conn.get_fsid())
             with conn.open_ioctx(self.pool) as ioctx:
                 order = int(math.log(self.WRITE_CHUNKSIZE, 2))
                 LOG.debug('creating image %s with order %d and size %d',
                           image_name, order, image_size)
                 if image_size == 0:
-                    LOG.warning(_("since image size is zero we will be doing "
-                                  "resize-before-write for each chunk which "
-                                  "will be considerably slower than normal"))
+                    LOG.warning(_LW("Since image size is zero we will be "
+                                    "doing resize-before-write which will be "
+                                    "slower than normal"))
 
                 try:
                     loc = self._create_image(fsid, conn, ioctx, image_name,
@@ -493,27 +573,45 @@ class Store(driver.Store):
                         chunks = utils.chunkreadable(image_file,
                                                      self.WRITE_CHUNKSIZE)
                         for chunk in chunks:
-                            # If the image size provided is zero we need to do
-                            # a resize for the amount we are writing. This will
-                            # be slower so setting a higher chunk size may
-                            # speed things up a bit.
-                            if image_size == 0:
-                                chunk_length = len(chunk)
-                                length = offset + chunk_length
-                                bytes_written += chunk_length
-                                LOG.debug(_("resizing image to %s KiB") %
-                                          (length / units.Ki))
-                                image.resize(length)
-                            LOG.debug(_("writing chunk at offset %s") %
-                                      (offset))
-                            offset += image.write(chunk, offset)
+                            # NOTE(jokke): If we don't know image size we need
+                            # to resize it on write. The resize amount will
+                            # ramp up to 8 gigs.
+                            chunk_length = len(chunk)
+                            self.size = self._resize_on_write(image,
+                                                              image_size,
+                                                              bytes_written,
+                                                              chunk_length)
+                            bytes_written += chunk_length
+                            if not (self.thin_provisioning and not any(chunk)):
+                                image.write(chunk, offset)
+                            offset += chunk_length
                             os_hash_value.update(chunk)
                             checksum.update(chunk)
                             if verifier:
                                 verifier.update(chunk)
+
+                        # Lets trim the image in case we overshoot with resize
+                        if image_size == 0:
+                            image.resize(bytes_written)
+
                         if loc.snapshot:
                             image.create_snap(loc.snapshot)
                             image.protect_snap(loc.snapshot)
+                except rbd.NoSpace:
+                    log_msg = (_LE("Failed to store image %(img_name)s "
+                                   "insufficient space available") %
+                               {'img_name': image_name})
+                    LOG.error(log_msg)
+
+                    # Delete image if one was created
+                    try:
+                        target_pool = loc.pool or self.pool
+                        self._delete_image(target_pool, loc.image,
+                                           loc.snapshot)
+                    except exceptions.NotFound:
+                        pass
+
+                    raise exceptions.StorageFull(message=log_msg)
                 except Exception as exc:
                     log_msg = (_LE("Failed to store image %(img_name)s "
                                    "Store Exception %(store_exc)s") %
@@ -538,7 +636,7 @@ class Store(driver.Store):
         # Add store backend information to location metadata
         metadata = {}
         if self.backend_group:
-            metadata['backend'] = u"%s" % self.backend_group
+            metadata['store'] = u"%s" % self.backend_group
 
         return (loc.get_uri(),
                 image_size,
