@@ -768,6 +768,7 @@ def Store(conf, backend=None):
         return MultiTenantStore(conf, backend=backend)
     return SingleTenantStore(conf, backend=backend)
 
+
 Store.OPTIONS = _SWIFT_OPTS + sutils.swift_opts + buffered.BUFFERING_OPTS
 
 
@@ -806,6 +807,10 @@ class BaseStore(driver.Store):
         self.insecure = glance_conf.swift_store_auth_insecure
         self.ssl_compression = glance_conf.swift_store_ssl_compression
         self.cacert = glance_conf.swift_store_cacert
+        if self.insecure:
+            self.ks_verify = False
+        else:
+            self.ks_verify = self.cacert or True
         if swiftclient is None:
             msg = _("Missing dependency python_swiftclient.")
             raise exceptions.BadStoreConfiguration(store_name="swift",
@@ -861,7 +866,7 @@ class BaseStore(driver.Store):
                     try:
                         return next(self.wrapped)
                     except StopIteration:
-                        return ''
+                        return b''
 
             length = int(resp_headers.get('content-length', 0))
             if allow_retry:
@@ -1030,11 +1035,12 @@ class BaseStore(driver.Store):
                     if image_size == 0:
                         image_size = combined_chunks_size
 
-                    # Now we write the object manifest and return the
-                    # manifest's etag...
+                    # Now we write the object manifest in X-Object-Manifest
+                    # header as defined for Dynamic Large Objects (DLO) Mode.
+                    # This request does not include ETag as PUT request has not
+                    # actual content that we need to verify.
                     manifest = "%s/%s-" % (location.container, location.obj)
-                    headers = {'ETag': hashlib.md5(b"").hexdigest(),
-                               'X-Object-Manifest': manifest}
+                    headers = {'X-Object-Manifest': manifest}
 
                     # The ETag returned for the manifest is actually the
                     # MD5 hash of the concatenated checksums of the strings
@@ -1059,7 +1065,7 @@ class BaseStore(driver.Store):
 
                 metadata = {}
                 if self.backend_group:
-                    metadata['backend'] = u"%s" % self.backend_group
+                    metadata['store'] = u"%s" % self.backend_group
 
                 return (location.get_uri(credentials_included=include_creds),
                         image_size, obj_etag, os_hash_value.hexdigest(),
@@ -1117,7 +1123,7 @@ class BaseStore(driver.Store):
                     try:
                         connection.delete_object(obj_container,
                                                  segment['name'])
-                    except swiftclient.ClientException as e:
+                    except swiftclient.ClientException:
                         msg = _('Unable to delete segment %(segment_name)s')
                         msg = msg % {'segment_name': segment['name']}
                         LOG.exception(msg)
@@ -1221,6 +1227,9 @@ class BaseStore(driver.Store):
         msg = _("There is no Connection Manager implemented for %s class.")
         raise NotImplementedError(msg % self.__class__.__name__)
 
+    def _set_url_prefix(self, context=None):
+        raise NotImplementedError()
+
 
 class SingleTenantStore(BaseStore):
     EXAMPLE_URL = "swift://<USER>:<KEY>@<AUTH_ADDRESS>/<CONTAINER>/<FILE>"
@@ -1279,6 +1288,50 @@ class SingleTenantStore(BaseStore):
             LOG.error(reason)
             raise exceptions.BadStoreConfiguration(store_name="swift",
                                                    reason=reason)
+        if self.backend_group:
+            self._set_url_prefix()
+
+    def _get_credstring(self):
+        if self.user and self.key:
+            return '%s:%s' % (urllib.parse.quote(self.user),
+                              urllib.parse.quote(self.key))
+        return ''
+
+    def _set_url_prefix(self, context=None):
+        auth_or_store_url = self.auth_address
+        if auth_or_store_url.startswith('http://'):
+            auth_or_store_url = auth_or_store_url[len('http://'):]
+        elif auth_or_store_url.startswith('https://'):
+            auth_or_store_url = auth_or_store_url[len('https://'):]
+
+        credstring = self._get_credstring()
+        auth_or_store_url = auth_or_store_url.strip('/')
+        container = self.container.strip('/')
+
+        if sutils.is_multiple_swift_store_accounts_enabled(
+                self.conf, backend=self.backend_group):
+            include_creds = False
+        else:
+            include_creds = True
+
+        if not include_creds:
+            store = getattr(self.conf,
+                            self.backend_group).default_swift_reference
+
+            self._url_prefix = '%s://%s/%s/' % (
+                'swift+config', store, container)
+            return
+
+        if self.scheme == 'swift+config':
+            if self.ssl_enabled:
+                self.scheme = 'swift+https'
+            else:
+                self.scheme = 'swift+http'
+        if credstring != '':
+            credstring = "%s@" % credstring
+
+        self._url_prefix = '%s://%s%s/%s/' % (
+            self.scheme, credstring, auth_or_store_url, container)
 
     def create_location(self, image_id, context=None):
         container_name = self.get_container_name(image_id, self.container)
@@ -1406,7 +1459,7 @@ class SingleTenantStore(BaseStore):
             project_domain_id=self.project_domain_id,
             project_domain_name=self.project_domain_name)
 
-        sess = ks_session.Session(auth=password, verify=not self.insecure)
+        sess = ks_session.Session(auth=password, verify=self.ks_verify)
         return ks_client.Client(session=sess)
 
     def get_manager(self, store_location, context=None, allow_reauth=False):
@@ -1501,6 +1554,11 @@ class MultiTenantStore(BaseStore):
         return StoreLocation(specs, self.conf,
                              backend_group=self.backend_group)
 
+    def _set_url_prefix(self, context=None):
+        ep = self._get_endpoint(context)
+        self._url_prefix = "%s://%s:%s_" % (
+            self.scheme, ep, self.container)
+
     def get_connection(self, location, context=None):
         return swiftclient.Connection(
             preauthurl=location.swift_url,
@@ -1535,12 +1593,15 @@ class MultiTenantStore(BaseStore):
         project_domain_name = default_swift_reference.get(
             'project_domain_name')
 
+        if self.backend_group:
+            self._set_url_prefix(context=context)
+
         # create client for multitenant user(trustor)
         trustor_auth = ks_identity.V3Token(auth_url=auth_address,
                                            token=context.auth_token,
                                            project_id=context.tenant)
         trustor_sess = ks_session.Session(auth=trustor_auth,
-                                          verify=not self.insecure)
+                                          verify=self.ks_verify)
         trustor_client = ks_client.Client(session=trustor_sess)
         auth_ref = trustor_client.session.auth.get_auth_ref(trustor_sess)
         roles = [t['name'] for t in auth_ref['roles']]
@@ -1557,7 +1618,7 @@ class MultiTenantStore(BaseStore):
             project_domain_id=project_domain_id,
             project_domain_name=project_domain_name)
         trustee_sess = ks_session.Session(auth=password,
-                                          verify=not self.insecure)
+                                          verify=self.ks_verify)
         trustee_client = ks_client.Client(session=trustee_sess)
 
         # request glance user id - we will use it as trustee user
@@ -1584,7 +1645,7 @@ class MultiTenantStore(BaseStore):
         # now we can authenticate against KS
         # as trustee of user who provided token
         client_sess = ks_session.Session(auth=client_password,
-                                         verify=not self.insecure)
+                                         verify=self.ks_verify)
         return ks_client.Client(session=client_sess)
 
     def get_manager(self, store_location, context=None, allow_reauth=False):
@@ -1631,7 +1692,20 @@ class ChunkReader(object):
         if i > left:
             i = left
 
-        result = self.do_read(i)
+        # Note(rosmaita): under some circumstances in py3, a zero-byte
+        # read results in a non-byte value that then causes a "unicode
+        # objects must be encoded before hashing" error when we do the
+        # hash computations below.  (At least that seems to be what's
+        # happening in testing.)  So just fake a zero-byte read and let
+        # the current execution path continue.
+        # See https://bugs.launchpad.net/glance-store/+bug/1805332
+        # TODO(rosmaita): find what in the execution path is returning
+        # a native string instead of bytes and fix it.
+        if i == 0:
+            result = b''
+        else:
+            result = self.do_read(i)
+
         self.bytes_read += len(result)
         self.checksum.update(result)
         self.os_hash_value.update(result)
