@@ -100,14 +100,38 @@ Related options:
 ]
 
 _config_defaults = {'user_domain_id': 'default',
-                    'user_domain_name': None,
+                    'user_domain_name': 'default',
                     'project_domain_id': 'default',
-                    'project_domain_name': None}
+                    'project_domain_name': 'default'}
 
 if sys.version_info >= (3, 2):
-    CONFIG = configparser.ConfigParser(defaults=_config_defaults)
+    parser_class = configparser.ConfigParser
 else:
-    CONFIG = configparser.SafeConfigParser(defaults=_config_defaults)
+    parser_class = configparser.SafeConfigParser
+
+
+class SwiftConfigParser(parser_class):
+
+    def get(self, *args, **kwargs):
+        value = super(parser_class, self).get(*args, **kwargs)
+        return self._process_quotes(value)
+
+    @staticmethod
+    def _process_quotes(value):
+        if value:
+            if value[0] in "\"'":
+                if len(value) == 1 or value[-1] != value[0]:
+                    raise ValueError('Non-closed quote: %s' %
+                                     value)
+                value = value[1:-1]
+        return value
+
+
+if sys.version_info >= (3,):
+    CONFIG = SwiftConfigParser(defaults=_config_defaults)
+else:
+    CONFIG = parser_class(defaults=_config_defaults)
+
 LOG = logging.getLogger(__name__)
 
 
@@ -199,6 +223,6 @@ class SwiftParams(object):
                     reference['auth_version'] = av
 
                 account_params[ref] = reference
-            except (ValueError, SyntaxError, configparser.NoOptionError) as e:
+            except (ValueError, SyntaxError, configparser.NoOptionError):
                 LOG.exception(_LE("Invalid format of swift store config cfg"))
         return account_params
