@@ -463,8 +463,9 @@ class Store(glance_store.Store):
             raise exceptions.BadStoreConfiguration(
                 store_name='vmware_datastore', reason=msg)
         if len(parts) == 3 and parts[2]:
-            weight = parts[2]
-            if not weight.isdigit():
+            try:
+                weight = int(parts[2])
+            except ValueError:
                 msg = (_('Invalid weight value %(weight)s in '
                          'vmware_datastores configuration') %
                        {'weight': weight})
@@ -501,7 +502,7 @@ class Store(glance_store.Store):
                 LOG.error(msg)
                 raise exceptions.BadStoreConfiguration(
                     store_name='vmware_datastore', reason=msg)
-            ds_map.setdefault(int(weight), []).append(ds_obj)
+            ds_map.setdefault(weight, []).append(ds_obj)
         return ds_map
 
     def configure_add(self):
@@ -514,6 +515,19 @@ class Store(glance_store.Store):
             store_conf = self.conf.glance_store
 
         self.store_image_dir = store_conf.vmware_store_image_dir
+        if self.backend_group:
+            self._set_url_prefix()
+
+    def _set_url_prefix(self):
+        path = os.path.join(DS_URL_PREFIX,
+                            self.store_image_dir)
+
+        if netutils.is_valid_ipv6(self.server_host):
+            self._url_prefix = '%s://[%s]%s' % (self.scheme,
+                                                self.server_host, path)
+        else:
+            self._url_prefix = '%s://%s%s' % (self.scheme,
+                                              self.server_host, path)
 
     def select_datastore(self, image_size):
         """Select a datastore with free space larger than image size."""
@@ -643,7 +657,7 @@ class Store(glance_store.Store):
 
         metadata = {}
         if self.backend_group:
-            metadata['backend'] = u"%s" % self.backend_group
+            metadata['store'] = u"%s" % self.backend_group
 
         return (loc.get_uri(),
                 image_file.size,
