@@ -21,16 +21,20 @@ System-level utilities and helper functions.
 import logging
 import uuid
 
+from oslo_concurrency import lockutils
+
 try:
     from eventlet import sleep
 except ImportError:
     from time import sleep
-from oslo_utils import encodeutils
-import six
+
+from glance_store.i18n import _
 
 
 LOG = logging.getLogger(__name__)
 
+synchronized = lockutils.synchronized_with_prefix('glance_store-')
+
 
 def is_uuid_like(val):
     """Returns validation of a value as a UUID.
@@ -135,21 +139,7 @@ class CooperativeReader(object):
         try:
             return next(self.iterator)
         except StopIteration:
-            return ''
+            return b''
 
     def __iter__(self):
         return cooperative_iter(self.fd.__iter__())
-
-
-def exception_to_str(exc):
-    try:
-        error = six.text_type(exc)
-    except UnicodeError:
-        try:
-            error = str(exc)
-        except UnicodeError:
-            error = ("Caught '%(exception)s' exception." %
-                     {"exception": exc.__class__.__name__})
-    if six.PY2:
-        error = encodeutils.safe_encode(error, errors='ignore')
-    return error
