@@ -16,28 +16,33 @@
 import contextlib
 import errno
 import hashlib
-import mock
 import os
+from unittest import mock
+
 import six
 import socket
+import sys
 import tempfile
 import time
 import uuid
 
+from cinderclient.v3 import client as cinderclient
 from os_brick.initiator import connector
 from oslo_concurrency import processutils
 from oslo_utils import units
 
-from glance_store._drivers import cinder
 from glance_store import exceptions
 from glance_store import location
 from glance_store.tests import base
 from glance_store.tests.unit import test_store_capabilities
 
+sys.modules['glance_store.common.fs_mount'] = mock.Mock()
+from glance_store._drivers import cinder # noqa
+
 
 class FakeObject(object):
     def __init__(self, **kwargs):
-        for name, value in six.iteritems(kwargs):
+        for name, value in kwargs.items():
             setattr(self, name, value)
 
 
@@ -60,9 +65,10 @@ class TestCinderStore(base.StoreBaseTest,
                                   user='fake_user',
                                   auth_token='fake_token',
                                   tenant='fake_tenant')
+        self.hash_algo = 'sha256'
 
     def test_get_cinderclient(self):
-        cc = cinder.get_cinderclient(self.conf, self.context)
+        cc = self.store.get_cinderclient(self.context)
         self.assertEqual('fake_token', cc.client.auth_token)
         self.assertEqual('http://foo/public_url', cc.client.management_url)
 
@@ -71,10 +77,21 @@ class TestCinderStore(base.StoreBaseTest,
         self.config(cinder_store_password='test_password')
         self.config(cinder_store_project_name='test_project')
         self.config(cinder_store_auth_address='test_address')
-        cc = cinder.get_cinderclient(self.conf, self.context)
+        cc = self.store.get_cinderclient(self.context)
         self.assertIsNone(cc.client.auth_token)
         self.assertEqual('test_address', cc.client.management_url)
 
+    def test_get_cinderclient_with_user_overriden_and_region(self):
+        self.config(cinder_os_region_name='test_region')
+        fake_client = FakeObject(client=FakeObject(auth_token=None))
+        with mock.patch.object(cinderclient, 'Client',
+                               return_value=fake_client) as mock_client:
+            self.test_get_cinderclient_with_user_overriden()
+            mock_client.assert_called_once_with(
+                'test_user', 'test_password', 'test_project',
+                auth_url='test_address', cacert=None, insecure=False,
+                region_name='test_region', retries=3)
+
     def test_temporary_chown(self):
         class fake_stat(object):
             st_uid = 1
@@ -82,9 +99,9 @@ class TestCinderStore(base.StoreBaseTest,
         with mock.patch.object(os, 'stat', return_value=fake_stat()), \
                 mock.patch.object(os, 'getuid', return_value=2), \
                 mock.patch.object(processutils, 'execute') as mock_execute, \
-                mock.patch.object(cinder, 'get_root_helper',
+                mock.patch.object(cinder.Store, 'get_root_helper',
                                   return_value='sudo'):
-            with cinder.temporary_chown('test'):
+            with self.store.temporary_chown('test'):
                 pass
             expected_calls = [mock.call('chown', 2, 'test', run_as_root=True,
                                         root_helper='sudo'),
@@ -132,7 +149,11 @@ class TestCinderStore(base.StoreBaseTest,
                           volume_available, 'available', 'in-use')
         fake_manager.get.assert_called_with('fake-id')
 
-    def _test_open_cinder_volume(self, open_mode, attach_mode, error):
+    def _test_open_cinder_volume(self, open_mode, attach_mode, error,
+                                 multipath_supported=False,
+                                 enforce_multipath=False,
+                                 encrypted_nfs=False):
+        self.config(cinder_mount_point_base=None)
         fake_volume = mock.MagicMock(id=str(uuid.uuid4()), status='available')
         fake_volumes = FakeObject(get=lambda id: fake_volume,
                                   detach=mock.Mock())
@@ -144,7 +165,7 @@ class TestCinderStore(base.StoreBaseTest,
             disconnect_volume=mock.Mock())
 
         @contextlib.contextmanager
-        def fake_chown(path):
+        def fake_chown(path, backend=None):
             yield
 
         def do_open():
@@ -158,27 +179,50 @@ class TestCinderStore(base.StoreBaseTest,
                              kwargs['conn'])
             return fake_connector
 
+        root_helper = "sudo glance-rootwrap /etc/glance/rootwrap.conf"
         with mock.patch.object(cinder.Store,
                                '_wait_volume_status',
                                return_value=fake_volume), \
-                mock.patch.object(cinder, 'temporary_chown',
+                mock.patch.object(cinder.Store, 'temporary_chown',
                                   side_effect=fake_chown), \
-                mock.patch.object(cinder, 'get_root_helper'), \
-                mock.patch.object(connector, 'get_connector_properties'), \
+                mock.patch.object(cinder.Store, 'get_root_helper',
+                                  return_value=root_helper), \
                 mock.patch.object(connector.InitiatorConnector, 'factory',
                                   side_effect=fake_factory):
 
-            if error:
-                self.assertRaises(error, do_open)
-            else:
-                do_open()
+            with mock.patch.object(connector,
+                                   'get_connector_properties') as mock_conn:
+                if error:
+                    self.assertRaises(error, do_open)
+                elif encrypted_nfs:
+                    fake_volume.initialize_connection.return_value = {
+                        'driver_volume_type': 'nfs'
+                    }
+                    fake_volume.encrypted = True
+                    try:
+                        with self.store._open_cinder_volume(
+                                fake_client, fake_volume, open_mode):
+                            pass
+                    except exceptions.BackendException:
+                        self.assertEqual(1,
+                                         fake_volume.unreserve.call_count)
+                        self.assertEqual(1,
+                                         fake_volume.delete.call_count)
+                else:
+                    do_open()
 
-            fake_connector.connect_volume.assert_called_once_with(mock.ANY)
-            fake_connector.disconnect_volume.assert_called_once_with(
-                mock.ANY, fake_devinfo)
-            fake_volume.attach.assert_called_once_with(
-                None, None, attach_mode, host_name=socket.gethostname())
-            fake_volumes.detach.assert_called_once_with(fake_volume)
+                if not encrypted_nfs:
+                    mock_conn.assert_called_once_with(
+                        root_helper, socket.gethostname(),
+                        multipath_supported, enforce_multipath)
+                    fake_connector.connect_volume.assert_called_once_with(
+                        mock.ANY)
+                    fake_connector.disconnect_volume.assert_called_once_with(
+                        mock.ANY, fake_devinfo)
+                    fake_volume.attach.assert_called_once_with(
+                        None, 'glance_store', attach_mode,
+                        host_name=socket.gethostname())
+                    fake_volumes.detach.assert_called_once_with(fake_volume)
 
     def test_open_cinder_volume_rw(self):
         self._test_open_cinder_volume('wb', 'rw', None)
@@ -189,6 +233,21 @@ class TestCinderStore(base.StoreBaseTest,
     def test_open_cinder_volume_error(self):
         self._test_open_cinder_volume('wb', 'rw', IOError)
 
+    def test_open_cinder_volume_multipath_supported(self):
+        self.config(cinder_use_multipath=True)
+        self._test_open_cinder_volume('wb', 'rw', None,
+                                      multipath_supported=True)
+
+    def test_open_cinder_volume_enforce_multipath(self):
+        self.config(cinder_use_multipath=True)
+        self.config(cinder_enforce_multipath=True)
+        self._test_open_cinder_volume('wb', 'rw', None,
+                                      multipath_supported=True,
+                                      enforce_multipath=True)
+
+    def test_open_cinder_volume_nfs_encrypted(self):
+        self._test_open_cinder_volume('rb', 'ro', None, encrypted_nfs=True)
+
     def test_cinder_configure_add(self):
         self.assertRaises(exceptions.BadStoreConfiguration,
                           self.store._check_context, None)
@@ -216,7 +275,7 @@ class TestCinderStore(base.StoreBaseTest,
             self.assertEqual('rb', mode)
             yield volume_file
 
-        with mock.patch.object(cinder, 'get_cinderclient') as mock_cc, \
+        with mock.patch.object(cinder.Store, 'get_cinderclient') as mock_cc, \
                 mock.patch.object(self.store, '_open_cinder_volume',
                                   side_effect=fake_open):
             mock_cc.return_value = FakeObject(client=fake_client,
@@ -242,7 +301,7 @@ class TestCinderStore(base.StoreBaseTest,
         fake_volume = FakeObject(size=5, metadata={})
         fake_volumes = {fake_volume_uuid: fake_volume}
 
-        with mock.patch.object(cinder, 'get_cinderclient') as mocked_cc:
+        with mock.patch.object(cinder.Store, 'get_cinderclient') as mocked_cc:
             mocked_cc.return_value = FakeObject(client=fake_client,
                                                 volumes=fake_volumes)
 
@@ -259,7 +318,7 @@ class TestCinderStore(base.StoreBaseTest,
                                  metadata={'image_size': expected_image_size})
         fake_volumes = {fake_volume_uuid: fake_volume}
 
-        with mock.patch.object(cinder, 'get_cinderclient') as mocked_cc:
+        with mock.patch.object(cinder.Store, 'get_cinderclient') as mocked_cc:
             mocked_cc.return_value = FakeObject(client=fake_client,
                                                 volumes=fake_volumes)
 
@@ -275,43 +334,49 @@ class TestCinderStore(base.StoreBaseTest,
         expected_file_contents = b"*" * expected_size
         image_file = six.BytesIO(expected_file_contents)
         expected_checksum = hashlib.md5(expected_file_contents).hexdigest()
+        expected_multihash = hashlib.sha256(expected_file_contents).hexdigest()
         expected_location = 'cinder://%s' % fake_volume.id
         fake_client = FakeObject(auth_token=None, management_url=None)
         fake_volume.manager.get.return_value = fake_volume
         fake_volumes = FakeObject(create=mock.Mock(return_value=fake_volume))
+        self.config(cinder_volume_type='some_type')
 
         @contextlib.contextmanager
         def fake_open(client, volume, mode):
             self.assertEqual('wb', mode)
             yield volume_file
 
-        with mock.patch.object(cinder, 'get_cinderclient') as mock_cc, \
+        with mock.patch.object(cinder.Store, 'get_cinderclient') as mock_cc, \
                 mock.patch.object(self.store, '_open_cinder_volume',
                                   side_effect=fake_open):
             mock_cc.return_value = FakeObject(client=fake_client,
                                               volumes=fake_volumes)
-            loc, size, checksum, _ = self.store.add(expected_image_id,
-                                                    image_file,
-                                                    expected_size,
-                                                    self.context,
-                                                    verifier)
+            loc, size, checksum, multihash, _ = self.store.add(
+                expected_image_id, image_file, expected_size, self.hash_algo,
+                self.context, verifier)
             self.assertEqual(expected_location, loc)
             self.assertEqual(expected_size, size)
             self.assertEqual(expected_checksum, checksum)
+            self.assertEqual(expected_multihash, multihash)
             fake_volumes.create.assert_called_once_with(
                 1,
                 name='image-%s' % expected_image_id,
                 metadata={'image_owner': self.context.tenant,
                           'glance_image_id': expected_image_id,
-                          'image_size': str(expected_size)})
+                          'image_size': str(expected_size)},
+                volume_type='some_type')
 
     def test_cinder_add(self):
-        fake_volume = mock.MagicMock(id=str(uuid.uuid4()), status='available')
+        fake_volume = mock.MagicMock(id=str(uuid.uuid4()),
+                                     status='available',
+                                     size=1)
         volume_file = six.BytesIO()
         self._test_cinder_add(fake_volume, volume_file)
 
     def test_cinder_add_with_verifier(self):
-        fake_volume = mock.MagicMock(id=str(uuid.uuid4()), status='available')
+        fake_volume = mock.MagicMock(id=str(uuid.uuid4()),
+                                     status='available',
+                                     size=1)
         volume_file = six.BytesIO()
         verifier = mock.MagicMock()
         self._test_cinder_add(fake_volume, volume_file, 1, verifier)
@@ -321,7 +386,9 @@ class TestCinderStore(base.StoreBaseTest,
         e = IOError()
         volume_file = six.BytesIO()
         e.errno = errno.ENOSPC
-        fake_volume = mock.MagicMock(id=str(uuid.uuid4()), status='available')
+        fake_volume = mock.MagicMock(id=str(uuid.uuid4()),
+                                     status='available',
+                                     size=1)
         with mock.patch.object(volume_file, 'write', side_effect=e):
             self.assertRaises(exceptions.StorageFull,
                               self._test_cinder_add, fake_volume, volume_file)
@@ -333,7 +400,7 @@ class TestCinderStore(base.StoreBaseTest,
         fake_volume = FakeObject(delete=mock.Mock())
         fake_volumes = {fake_volume_uuid: fake_volume}
 
-        with mock.patch.object(cinder, 'get_cinderclient') as mocked_cc:
+        with mock.patch.object(cinder.Store, 'get_cinderclient') as mocked_cc:
             mocked_cc.return_value = FakeObject(client=fake_client,
                                                 volumes=fake_volumes)
 
@@ -341,3 +408,6 @@ class TestCinderStore(base.StoreBaseTest,
             loc = location.get_location_from_uri(uri, conf=self.conf)
             self.store.delete(loc, context=self.context)
             fake_volume.delete.assert_called_once_with()
+
+    def test_set_url_prefix(self):
+        self.assertEqual('cinder://', self.store._url_prefix)
