@@ -18,9 +18,9 @@
 import errno
 import hashlib
 import json
-import mock
 import os
 import stat
+from unittest import mock
 import uuid
 
 import fixtures
@@ -30,8 +30,7 @@ from six.moves import builtins
 # NOTE(jokke): simplified transition to py3, behaves like py2 xrange
 from six.moves import range
 
-from glance_store._drivers.filesystem import ChunkedFile
-from glance_store._drivers.filesystem import Store
+from glance_store._drivers import filesystem
 from glance_store import exceptions
 from glance_store import location
 from glance_store.tests import base
@@ -44,19 +43,14 @@ class TestStore(base.StoreBaseTest,
     def setUp(self):
         """Establish a clean test environment."""
         super(TestStore, self).setUp()
-        self.orig_chunksize = Store.READ_CHUNKSIZE
-        Store.READ_CHUNKSIZE = 10
-        self.store = Store(self.conf)
+        self.store = filesystem.Store(self.conf)
         self.config(filesystem_store_datadir=self.test_dir,
+                    filesystem_store_chunk_size=10,
                     stores=['glance.store.filesystem.Store'],
                     group="glance_store")
         self.store.configure()
         self.register_store_schemes(self.store, 'file')
-
-    def tearDown(self):
-        """Clear the test environment."""
-        super(TestStore, self).tearDown()
-        ChunkedFile.CHUNKSIZE = self.orig_chunksize
+        self.hash_algo = 'sha256'
 
     def _create_metadata_json_file(self, metadata):
         expected_image_id = str(uuid.uuid4())
@@ -75,7 +69,7 @@ class TestStore(base.StoreBaseTest,
         image_file = six.BytesIO(expected_file_contents)
         self.store.FILESYSTEM_STORE_METADATA = in_metadata
         return self.store.add(expected_image_id, image_file,
-                              expected_file_size)
+                              expected_file_size, self.hash_algo)
 
     def test_get(self):
         """Test a "normal" retrieval of an image in chunks."""
@@ -84,9 +78,8 @@ class TestStore(base.StoreBaseTest,
         file_contents = b"chunk00000remainder"
         image_file = six.BytesIO(file_contents)
 
-        loc, size, checksum, _ = self.store.add(image_id,
-                                                image_file,
-                                                len(file_contents))
+        loc, size, checksum, multihash, _ = self.store.add(
+            image_id, image_file, len(file_contents), self.hash_algo)
 
         # Now read it back...
         uri = "file:///%s/%s" % (self.test_dir, image_id)
@@ -111,9 +104,8 @@ class TestStore(base.StoreBaseTest,
         file_contents = b"chunk00000remainder"
         image_file = six.BytesIO(file_contents)
 
-        loc, size, checksum, _ = self.store.add(image_id,
-                                                image_file,
-                                                len(file_contents))
+        loc, size, checksum, multihash, _ = self.store.add(
+            image_id, image_file, len(file_contents), self.hash_algo)
 
         # Now read it back...
         uri = "file:///%s/%s" % (self.test_dir, image_id)
@@ -151,24 +143,30 @@ class TestStore(base.StoreBaseTest,
                           self.store.get,
                           loc)
 
-    def test_add(self):
+    def _do_test_add(self, enable_thin_provisoning):
         """Test that we can add an image via the filesystem backend."""
-        ChunkedFile.CHUNKSIZE = units.Ki
+        self.config(filesystem_store_chunk_size=units.Ki,
+                    filesystem_thin_provisioning=enable_thin_provisoning,
+                    group='glance_store')
+        self.store.configure()
+
+        filesystem.ChunkedFile.CHUNKSIZE = units.Ki
         expected_image_id = str(uuid.uuid4())
         expected_file_size = 5 * units.Ki  # 5K
         expected_file_contents = b"*" * expected_file_size
         expected_checksum = hashlib.md5(expected_file_contents).hexdigest()
+        expected_multihash = hashlib.sha256(expected_file_contents).hexdigest()
         expected_location = "file://%s/%s" % (self.test_dir,
                                               expected_image_id)
         image_file = six.BytesIO(expected_file_contents)
 
-        loc, size, checksum, _ = self.store.add(expected_image_id,
-                                                image_file,
-                                                expected_file_size)
+        loc, size, checksum, multihash, _ = self.store.add(
+            expected_image_id, image_file, expected_file_size, self.hash_algo)
 
         self.assertEqual(expected_location, loc)
         self.assertEqual(expected_file_size, size)
         self.assertEqual(expected_checksum, checksum)
+        self.assertEqual(expected_multihash, multihash)
 
         uri = "file:///%s/%s" % (self.test_dir, expected_image_id)
         loc = location.get_location_from_uri(uri, conf=self.conf)
@@ -183,35 +181,122 @@ class TestStore(base.StoreBaseTest,
         self.assertEqual(expected_file_contents, new_image_contents)
         self.assertEqual(expected_file_size, new_image_file_size)
 
+    def test_thin_provisioning_is_disabled_by_default(self):
+        self.assertEqual(self.store.thin_provisioning, False)
+
+    def test_add_with_thick_provisioning(self):
+        self._do_test_add(enable_thin_provisoning=False)
+
+    def test_add_with_thin_provisioning(self):
+        self._do_test_add(enable_thin_provisoning=True)
+
+    def test_add_thick_provisioning_with_holes_in_file(self):
+        """
+        Tests that a file which contains null bytes chunks is fully
+        written with a thick provisioning configuration.
+        """
+        chunk_size = units.Ki  # 1K
+        content = b"*" * chunk_size + b"\x00" * chunk_size + b"*" * chunk_size
+        self._do_test_thin_provisioning(content, 3 * chunk_size, 0, 3, False)
+
+    def test_add_thin_provisioning_with_holes_in_file(self):
+        """
+        Tests that a file which contains null bytes chunks is sparsified
+        with a thin provisioning configuration.
+        """
+        chunk_size = units.Ki  # 1K
+        content = b"*" * chunk_size + b"\x00" * chunk_size + b"*" * chunk_size
+        self._do_test_thin_provisioning(content, 3 * chunk_size, 1, 2, True)
+
+    def test_add_thick_provisioning_without_holes_in_file(self):
+        """
+        Tests that a file which not contain null bytes chunks is fully
+        written with a thick provisioning configuration.
+        """
+        chunk_size = units.Ki  # 1K
+        content = b"*" * 3 * chunk_size
+        self._do_test_thin_provisioning(content, 3 * chunk_size, 0, 3, False)
+
+    def test_add_thin_provisioning_without_holes_in_file(self):
+        """
+        Tests that a file which not contain null bytes chunks is fully
+        written with a thin provisioning configuration.
+        """
+        chunk_size = units.Ki  # 1K
+        content = b"*" * 3 * chunk_size
+        self._do_test_thin_provisioning(content, 3 * chunk_size, 0, 3, True)
+
+    def test_add_thick_provisioning_with_partial_holes_in_file(self):
+        """
+        Tests that a file which contains null bytes not aligned with
+        chunk size is fully written with a thick provisioning configuration.
+        """
+        chunk_size = units.Ki  # 1K
+        my_chunk = int(chunk_size * 1.5)
+        content = b"*" * my_chunk + b"\x00" * my_chunk + b"*" * my_chunk
+        self._do_test_thin_provisioning(content, 3 * my_chunk, 0, 5, False)
+
+    def test_add_thin_provisioning_with_partial_holes_in_file(self):
+        """
+        Tests that a file which contains null bytes not aligned with
+        chunk size is sparsified with a thin provisioning configuration.
+        """
+        chunk_size = units.Ki  # 1K
+        my_chunk = int(chunk_size * 1.5)
+        content = b"*" * my_chunk + b"\x00" * my_chunk + b"*" * my_chunk
+        self._do_test_thin_provisioning(content, 3 * my_chunk, 1, 4, True)
+
+    def _do_test_thin_provisioning(self, content, size, truncate, write, thin):
+        self.config(filesystem_store_chunk_size=units.Ki,
+                    filesystem_thin_provisioning=thin,
+                    group='glance_store')
+        self.store.configure()
+
+        image_file = six.BytesIO(content)
+        image_id = str(uuid.uuid4())
+        with mock.patch.object(builtins, 'open') as popen:
+            self.store.add(image_id, image_file, size, self.hash_algo)
+            write_count = popen.return_value.__enter__().write.call_count
+            truncate_count = popen.return_value.__enter__().truncate.call_count
+            self.assertEqual(write_count, write)
+            self.assertEqual(truncate_count, truncate)
+
     def test_add_with_verifier(self):
         """Test that 'verifier.update' is called when verifier is provided."""
         verifier = mock.MagicMock(name='mock_verifier')
-        self.store.chunk_size = units.Ki
+        self.config(filesystem_store_chunk_size=units.Ki,
+                    group='glance_store')
+        self.store.configure()
+
         image_id = str(uuid.uuid4())
         file_size = units.Ki  # 1K
         file_contents = b"*" * file_size
         image_file = six.BytesIO(file_contents)
 
-        self.store.add(image_id, image_file, file_size, verifier=verifier)
+        self.store.add(image_id, image_file, file_size, self.hash_algo,
+                       verifier=verifier)
 
         verifier.update.assert_called_with(file_contents)
 
     def test_add_check_metadata_with_invalid_mountpoint_location(self):
         in_metadata = [{'id': 'abcdefg',
                        'mountpoint': '/xyz/images'}]
-        location, size, checksum, metadata = self._store_image(in_metadata)
+        location, size, checksum, multihash, metadata = self._store_image(
+            in_metadata)
         self.assertEqual({}, metadata)
 
     def test_add_check_metadata_list_with_invalid_mountpoint_locations(self):
         in_metadata = [{'id': 'abcdefg', 'mountpoint': '/xyz/images'},
                        {'id': 'xyz1234', 'mountpoint': '/pqr/images'}]
-        location, size, checksum, metadata = self._store_image(in_metadata)
+        location, size, checksum, multihash, metadata = self._store_image(
+            in_metadata)
         self.assertEqual({}, metadata)
 
     def test_add_check_metadata_list_with_valid_mountpoint_locations(self):
         in_metadata = [{'id': 'abcdefg', 'mountpoint': '/tmp'},
                        {'id': 'xyz1234', 'mountpoint': '/xyz'}]
-        location, size, checksum, metadata = self._store_image(in_metadata)
+        location, size, checksum, multihash, metadata = self._store_image(
+            in_metadata)
         self.assertEqual(in_metadata[0], metadata)
 
     def test_add_check_metadata_bad_nosuch_file(self):
@@ -225,9 +310,8 @@ class TestStore(base.StoreBaseTest,
         expected_file_contents = b"*" * expected_file_size
         image_file = six.BytesIO(expected_file_contents)
 
-        location, size, checksum, metadata = self.store.add(expected_image_id,
-                                                            image_file,
-                                                            expected_file_size)
+        location, size, checksum, multihash, metadata = self.store.add(
+            expected_image_id, image_file, expected_file_size, self.hash_algo)
 
         self.assertEqual(metadata, {})
 
@@ -236,22 +320,21 @@ class TestStore(base.StoreBaseTest,
         Tests that adding an image with an existing identifier
         raises an appropriate exception
         """
-        ChunkedFile.CHUNKSIZE = units.Ki
+        filesystem.ChunkedFile.CHUNKSIZE = units.Ki
         image_id = str(uuid.uuid4())
         file_size = 5 * units.Ki  # 5K
         file_contents = b"*" * file_size
         image_file = six.BytesIO(file_contents)
 
-        location, size, checksum, _ = self.store.add(image_id,
-                                                     image_file,
-                                                     file_size)
+        location, size, checksum, multihash, _ = self.store.add(
+            image_id, image_file, file_size, self.hash_algo)
         image_file = six.BytesIO(b"nevergonnamakeit")
         self.assertRaises(exceptions.Duplicate,
                           self.store.add,
-                          image_id, image_file, 0)
+                          image_id, image_file, 0, self.hash_algo)
 
     def _do_test_add_write_failure(self, errno, exception):
-        ChunkedFile.CHUNKSIZE = units.Ki
+        filesystem.ChunkedFile.CHUNKSIZE = units.Ki
         image_id = str(uuid.uuid4())
         file_size = 5 * units.Ki  # 5K
         file_contents = b"*" * file_size
@@ -265,7 +348,7 @@ class TestStore(base.StoreBaseTest,
 
             self.assertRaises(exception,
                               self.store.add,
-                              image_id, image_file, 0)
+                              image_id, image_file, 0, self.hash_algo)
             self.assertFalse(os.path.exists(path))
 
     def test_add_storage_full(self):
@@ -302,7 +385,7 @@ class TestStore(base.StoreBaseTest,
         Tests the partial image file is cleaned up after a read
         failure.
         """
-        ChunkedFile.CHUNKSIZE = units.Ki
+        filesystem.ChunkedFile.CHUNKSIZE = units.Ki
         image_id = str(uuid.uuid4())
         file_size = 5 * units.Ki  # 5K
         file_contents = b"*" * file_size
@@ -317,7 +400,7 @@ class TestStore(base.StoreBaseTest,
 
             self.assertRaises(AttributeError,
                               self.store.add,
-                              image_id, image_file, 0)
+                              image_id, image_file, 0, self.hash_algo)
             self.assertFalse(os.path.exists(path))
 
     def test_delete(self):
@@ -330,9 +413,8 @@ class TestStore(base.StoreBaseTest,
         file_contents = b"*" * file_size
         image_file = six.BytesIO(file_contents)
 
-        loc, size, checksum, _ = self.store.add(image_id,
-                                                image_file,
-                                                file_size)
+        loc, size, checksum, multihash, _ = self.store.add(
+            image_id, image_file, file_size, self.hash_algo)
 
         # Now check that we can delete it
         uri = "file:///%s/%s" % (self.test_dir, image_id)
@@ -363,9 +445,8 @@ class TestStore(base.StoreBaseTest,
         file_contents = b"*" * file_size
         image_file = six.BytesIO(file_contents)
 
-        loc, size, checksum, _ = self.store.add(image_id,
-                                                image_file,
-                                                file_size)
+        loc, size, checksum, multihash, _ = self.store.add(
+            image_id, image_file, file_size, self.hash_algo)
 
         uri = "file:///%s/%s" % (self.test_dir, image_id)
         loc = location.get_location_from_uri(uri, conf=self.conf)
@@ -519,22 +600,23 @@ class TestStore(base.StoreBaseTest,
             self.fail("configure() raised BadStoreConfiguration unexpectedly!")
 
         # Test that we can add an image via the filesystem backend
-        ChunkedFile.CHUNKSIZE = 1024
+        filesystem.ChunkedFile.CHUNKSIZE = 1024
         expected_image_id = str(uuid.uuid4())
         expected_file_size = 5 * units.Ki  # 5K
         expected_file_contents = b"*" * expected_file_size
         expected_checksum = hashlib.md5(expected_file_contents).hexdigest()
+        expected_multihash = hashlib.sha256(expected_file_contents).hexdigest()
         expected_location = "file://%s/%s" % (store_map[1],
                                               expected_image_id)
         image_file = six.BytesIO(expected_file_contents)
 
-        loc, size, checksum, _ = self.store.add(expected_image_id,
-                                                image_file,
-                                                expected_file_size)
+        loc, size, checksum, multihash, _ = self.store.add(
+            expected_image_id, image_file, expected_file_size, self.hash_algo)
 
         self.assertEqual(expected_location, loc)
         self.assertEqual(expected_file_size, size)
         self.assertEqual(expected_checksum, checksum)
+        self.assertEqual(expected_multihash, multihash)
 
         loc = location.get_location_from_uri(expected_location,
                                              conf=self.conf)
@@ -565,22 +647,23 @@ class TestStore(base.StoreBaseTest,
         self.store.configure()
 
         # Test that we can add an image via the filesystem backend
-        ChunkedFile.CHUNKSIZE = units.Ki
+        filesystem.ChunkedFile.CHUNKSIZE = units.Ki
         expected_image_id = str(uuid.uuid4())
         expected_file_size = 5 * units.Ki  # 5K
         expected_file_contents = b"*" * expected_file_size
         expected_checksum = hashlib.md5(expected_file_contents).hexdigest()
+        expected_multihash = hashlib.sha256(expected_file_contents).hexdigest()
         expected_location = "file://%s/%s" % (store_map[1],
                                               expected_image_id)
         image_file = six.BytesIO(expected_file_contents)
 
-        loc, size, checksum, _ = self.store.add(expected_image_id,
-                                                image_file,
-                                                expected_file_size)
+        loc, size, checksum, multihash, _ = self.store.add(
+            expected_image_id, image_file, expected_file_size, self.hash_algo)
 
         self.assertEqual(expected_location, loc)
         self.assertEqual(expected_file_size, size)
         self.assertEqual(expected_checksum, checksum)
+        self.assertEqual(expected_multihash, multihash)
 
         loc = location.get_location_from_uri(expected_location,
                                              conf=self.conf)
@@ -618,15 +701,18 @@ class TestStore(base.StoreBaseTest,
         with mock.patch.object(self.store, '_get_capacity_info') as capacity:
             capacity.return_value = 0
 
-            ChunkedFile.CHUNKSIZE = units.Ki
+            filesystem.ChunkedFile.CHUNKSIZE = units.Ki
             expected_image_id = str(uuid.uuid4())
             expected_file_size = 5 * units.Ki  # 5K
             expected_file_contents = b"*" * expected_file_size
             image_file = six.BytesIO(expected_file_contents)
 
-            self.assertRaises(exceptions.StorageFull, self.store.add,
-                              expected_image_id, image_file,
-                              expected_file_size)
+            self.assertRaises(exceptions.StorageFull,
+                              self.store.add,
+                              expected_image_id,
+                              image_file,
+                              expected_file_size,
+                              self.hash_algo)
 
     def test_configure_add_with_file_perm(self):
         """
@@ -671,22 +757,23 @@ class TestStore(base.StoreBaseTest,
 
         self.store.configure_add()
 
-        Store.WRITE_CHUNKSIZE = units.Ki
+        filesystem.Store.WRITE_CHUNKSIZE = units.Ki
         expected_image_id = str(uuid.uuid4())
         expected_file_size = 5 * units.Ki  # 5K
         expected_file_contents = b"*" * expected_file_size
         expected_checksum = hashlib.md5(expected_file_contents).hexdigest()
+        expected_multihash = hashlib.sha256(expected_file_contents).hexdigest()
         expected_location = "file://%s/%s" % (store,
                                               expected_image_id)
         image_file = six.BytesIO(expected_file_contents)
 
-        location, size, checksum, _ = self.store.add(expected_image_id,
-                                                     image_file,
-                                                     expected_file_size)
+        location, size, checksum, multihash, _ = self.store.add(
+            expected_image_id, image_file, expected_file_size, self.hash_algo)
 
         self.assertEqual(expected_location, location)
         self.assertEqual(expected_file_size, size)
         self.assertEqual(expected_checksum, checksum)
+        self.assertEqual(expected_multihash, multihash)
 
         # -rwx--x--x for store directory
         self.assertEqual(0o711, stat.S_IMODE(os.stat(store)[stat.ST_MODE]))
@@ -712,22 +799,23 @@ class TestStore(base.StoreBaseTest,
 
         self.store.configure_add()
 
-        Store.WRITE_CHUNKSIZE = units.Ki
+        filesystem.Store.WRITE_CHUNKSIZE = units.Ki
         expected_image_id = str(uuid.uuid4())
         expected_file_size = 5 * units.Ki  # 5K
         expected_file_contents = b"*" * expected_file_size
         expected_checksum = hashlib.md5(expected_file_contents).hexdigest()
+        expected_multihash = hashlib.sha256(expected_file_contents).hexdigest()
         expected_location = "file://%s/%s" % (store,
                                               expected_image_id)
         image_file = six.BytesIO(expected_file_contents)
 
-        location, size, checksum, _ = self.store.add(expected_image_id,
-                                                     image_file,
-                                                     expected_file_size)
+        location, size, checksum, multihash, _ = self.store.add(
+            expected_image_id, image_file, expected_file_size, self.hash_algo)
 
         self.assertEqual(expected_location, location)
         self.assertEqual(expected_file_size, size)
         self.assertEqual(expected_checksum, checksum)
+        self.assertEqual(expected_multihash, multihash)
 
         # -rwx------ for store directory
         self.assertEqual(0o700, stat.S_IMODE(os.stat(store)[stat.ST_MODE]))
@@ -735,3 +823,14 @@ class TestStore(base.StoreBaseTest,
         mode = os.stat(expected_location[len('file:/'):])[stat.ST_MODE]
         perm = int(str(self.conf.glance_store.filesystem_store_file_perm), 8)
         self.assertEqual(perm, stat.S_IMODE(mode))
+
+    def test_configure_add_chunk_size(self):
+        # This definitely won't be the default
+        chunk_size = units.Gi
+        self.config(filesystem_store_chunk_size=chunk_size,
+                    group="glance_store")
+        self.store.configure_add()
+
+        self.assertEqual(chunk_size, self.store.chunk_size)
+        self.assertEqual(chunk_size, self.store.READ_CHUNKSIZE)
+        self.assertEqual(chunk_size, self.store.WRITE_CHUNKSIZE)
