@@ -16,10 +16,12 @@
 import contextlib
 import errno
 import hashlib
-import mock
 import os
+from unittest import mock
+
 import six
 import socket
+import sys
 import tempfile
 import time
 import uuid
@@ -31,12 +33,14 @@ from oslo_config import cfg
 from oslo_utils import units
 
 import glance_store as store
-from glance_store._drivers import cinder
 from glance_store import exceptions
 from glance_store import location
 from glance_store.tests import base
 from glance_store.tests.unit import test_store_capabilities as test_cap
 
+sys.modules['glance_store.common.fs_mount'] = mock.Mock()
+from glance_store._drivers import cinder # noqa
+
 
 class FakeObject(object):
     def __init__(self, **kwargs):
@@ -67,8 +71,8 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
 
         # Ensure stores + locations cleared
         location.SCHEME_TO_CLS_BACKEND_MAP = {}
-
         store.create_multi_stores(self.conf)
+
         self.addCleanup(setattr, location, 'SCHEME_TO_CLS_BACKEND_MAP',
                         dict())
         self.test_dir = self.useFixture(fixtures.TempDir()).path
@@ -88,10 +92,18 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
                                   user='fake_user',
                                   auth_token='fake_token',
                                   tenant='fake_tenant')
+        self.fake_admin_context = mock.MagicMock()
+        self.fake_admin_context.elevated.return_value = FakeObject(
+            service_catalog=fake_sc,
+            user='admin_user',
+            auth_token='admin_token',
+            tenant='admin_project')
+
+    def test_location_url_prefix_is_set(self):
+        self.assertEqual("cinder://cinder1", self.store.url_prefix)
 
     def test_get_cinderclient(self):
-        cc = cinder.get_cinderclient(self.conf, self.context,
-                                     backend="cinder1")
+        cc = self.store.get_cinderclient(self.context)
         self.assertEqual('fake_token', cc.client.auth_token)
         self.assertEqual('http://foo/public_url', cc.client.management_url)
 
@@ -100,11 +112,18 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
         self.config(cinder_store_password='test_password', group="cinder1")
         self.config(cinder_store_project_name='test_project', group="cinder1")
         self.config(cinder_store_auth_address='test_address', group="cinder1")
-        cc = cinder.get_cinderclient(self.conf, self.context,
-                                     backend="cinder1")
+        cc = self.store.get_cinderclient(self.context)
         self.assertIsNone(cc.client.auth_token)
         self.assertEqual('test_address', cc.client.management_url)
 
+    def test_get_cinderclient_legacy_update(self):
+        cc = self.store.get_cinderclient(self.fake_admin_context,
+                                         legacy_update=True)
+        self.assertEqual('admin_token', cc.client.auth_token)
+        self.assertEqual('admin_user', cc.client.user)
+        self.assertEqual('admin_project', cc.client.projectid)
+        self.assertEqual('http://foo/public_url', cc.client.management_url)
+
     def test_temporary_chown(self):
         class fake_stat(object):
             st_uid = 1
@@ -112,9 +131,9 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
         with mock.patch.object(os, 'stat', return_value=fake_stat()), \
                 mock.patch.object(os, 'getuid', return_value=2), \
                 mock.patch.object(processutils, 'execute') as mock_execute, \
-                mock.patch.object(cinder, 'get_root_helper',
+                mock.patch.object(cinder.Store, 'get_root_helper',
                                   return_value='sudo'):
-            with cinder.temporary_chown('test'):
+            with self.store.temporary_chown('test'):
                 pass
             expected_calls = [mock.call('chown', 2, 'test', run_as_root=True,
                                         root_helper='sudo'),
@@ -162,7 +181,10 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
                           volume_available, 'available', 'in-use')
         fake_manager.get.assert_called_with('fake-id')
 
-    def _test_open_cinder_volume(self, open_mode, attach_mode, error):
+    def _test_open_cinder_volume(self, open_mode, attach_mode, error,
+                                 multipath_supported=False,
+                                 enforce_multipath=False):
+        self.config(cinder_mount_point_base=None, group='cinder1')
         fake_volume = mock.MagicMock(id=str(uuid.uuid4()), status='available')
         fake_volumes = FakeObject(get=lambda id: fake_volume,
                                   detach=mock.Mock())
@@ -192,26 +214,30 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
         with mock.patch.object(cinder.Store,
                                '_wait_volume_status',
                                return_value=fake_volume), \
-                mock.patch.object(cinder, 'temporary_chown',
+                mock.patch.object(cinder.Store, 'temporary_chown',
                                   side_effect=fake_chown), \
-                mock.patch.object(cinder, 'get_root_helper',
+                mock.patch.object(cinder.Store, 'get_root_helper',
                                   return_value=root_helper), \
-                mock.patch.object(connector, 'get_connector_properties'), \
                 mock.patch.object(connector.InitiatorConnector, 'factory',
                                   side_effect=fake_factory):
 
-            if error:
-                self.assertRaises(error, do_open)
-            else:
-                do_open()
+            with mock.patch.object(connector,
+                                   'get_connector_properties') as mock_conn:
+                if error:
+                    self.assertRaises(error, do_open)
+                else:
+                    do_open()
 
-            fake_connector.connect_volume.assert_called_once_with(mock.ANY)
-            fake_connector.disconnect_volume.assert_called_once_with(
-                mock.ANY, fake_devinfo)
-            fake_volume.attach.assert_called_once_with(
-                None, 'glance_store', attach_mode,
-                host_name=socket.gethostname())
-            fake_volumes.detach.assert_called_once_with(fake_volume)
+                mock_conn.assert_called_once_with(
+                    root_helper, socket.gethostname(), multipath_supported,
+                    enforce_multipath)
+                fake_connector.connect_volume.assert_called_once_with(mock.ANY)
+                fake_connector.disconnect_volume.assert_called_once_with(
+                    mock.ANY, fake_devinfo)
+                fake_volume.attach.assert_called_once_with(
+                    None, 'glance_store', attach_mode,
+                    host_name=socket.gethostname())
+                fake_volumes.detach.assert_called_once_with(fake_volume)
 
     def test_open_cinder_volume_rw(self):
         self._test_open_cinder_volume('wb', 'rw', None)
@@ -222,7 +248,20 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
     def test_open_cinder_volume_error(self):
         self._test_open_cinder_volume('wb', 'rw', IOError)
 
-    def test_cinder_configure_add(self):
+    def test_open_cinder_volume_multipath_disabled(self):
+        self.config(cinder_use_multipath=False, group='cinder1')
+        self._test_open_cinder_volume('wb', 'rw', None,
+                                      multipath_supported=False)
+
+    def test_open_cinder_volume_enforce_multipath(self):
+
+        self.config(cinder_use_multipath=True, group='cinder1')
+        self.config(cinder_enforce_multipath=True, group='cinder1')
+        self._test_open_cinder_volume('wb', 'rw', None,
+                                      multipath_supported=True,
+                                      enforce_multipath=True)
+
+    def test_cinder_check_context(self):
         self.assertRaises(exceptions.BadStoreConfiguration,
                           self.store._check_context, None)
 
@@ -232,6 +271,57 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
 
         self.store._check_context(FakeObject(service_catalog='fake'))
 
+    def test_cinder_configure_add(self):
+        with mock.patch.object(self.store, 'get_cinderclient') as mocked_cc:
+            def raise_(ex):
+                raise ex
+            mocked_cc.return_value = FakeObject(volume_types=FakeObject(
+                find=lambda name: 'some_type' if name == 'some_type'
+                else raise_(cinder.cinder_exception.NotFound(code=404))))
+            self.config(cinder_volume_type='some_type',
+                        group=self.store.backend_group)
+            # If volume type exists, no exception is raised
+            self.store.configure_add()
+            # setting cinder_volume_type to non-existent value will raise
+            # BadStoreConfiguration exception
+            self.config(cinder_volume_type='some_random_type',
+                        group=self.store.backend_group)
+
+            self.assertRaises(exceptions.BadStoreConfiguration,
+                              self.store.configure_add)
+            # when only 1 store is configured, BackendException is raised
+            self.config(enabled_backends={'cinder1': 'cinder'})
+            self.assertRaises(exceptions.BackendException,
+                              self.store.configure_add)
+
+    def test_is_image_associated_with_store(self):
+        with mock.patch.object(self.store, 'get_cinderclient') as mocked_cc:
+            mocked_cc.return_value = FakeObject(volumes=FakeObject(
+                get=lambda volume_id: FakeObject(volume_type='some_type')),
+                volume_types=FakeObject(
+                    default=lambda: FakeObject(name='some_type')))
+            # When cinder_volume_type is set and is same as volume's type
+            self.config(cinder_volume_type='some_type',
+                        group=self.store.backend_group)
+            fake_vol_id = str(uuid.uuid4())
+            type_match = self.store.is_image_associated_with_store(
+                self.context, fake_vol_id)
+            self.assertTrue(type_match)
+            # When cinder_volume_type is not set and volume's type is same as
+            # set default volume type
+            self.config(cinder_volume_type=None,
+                        group=self.store.backend_group)
+            type_match = self.store.is_image_associated_with_store(
+                self.context, fake_vol_id)
+            self.assertTrue(type_match)
+            # When cinder_volume_type is not set and volume's type does not
+            # match with default volume type
+            mocked_cc.return_value.volume_types = FakeObject(
+                default=lambda: {'name': 'random_type'})
+            type_match = self.store.is_image_associated_with_store(
+                self.context, fake_vol_id)
+            self.assertFalse(type_match)
+
     def test_cinder_get(self):
         expected_size = 5 * units.Ki
         expected_file_contents = b"*" * expected_size
@@ -249,12 +339,12 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
             self.assertEqual('rb', mode)
             yield volume_file
 
-        with mock.patch.object(cinder, 'get_cinderclient') as mock_cc, \
+        with mock.patch.object(cinder.Store, 'get_cinderclient') as mock_cc, \
                 mock.patch.object(self.store, '_open_cinder_volume',
                                   side_effect=fake_open):
             mock_cc.return_value = FakeObject(client=fake_client,
                                               volumes=fake_volumes)
-            uri = "cinder://%s" % fake_volume_uuid
+            uri = "cinder://cinder1/%s" % fake_volume_uuid
             loc = location.get_location_from_uri_and_backend(uri,
                                                              "cinder1",
                                                              conf=self.conf)
@@ -277,11 +367,11 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
         fake_volume = FakeObject(size=5, metadata={})
         fake_volumes = {fake_volume_uuid: fake_volume}
 
-        with mock.patch.object(cinder, 'get_cinderclient') as mocked_cc:
+        with mock.patch.object(cinder.Store, 'get_cinderclient') as mocked_cc:
             mocked_cc.return_value = FakeObject(client=fake_client,
                                                 volumes=fake_volumes)
 
-            uri = 'cinder://%s' % fake_volume_uuid
+            uri = 'cinder://cinder1/%s' % fake_volume_uuid
             loc = location.get_location_from_uri_and_backend(uri,
                                                              "cinder1",
                                                              conf=self.conf)
@@ -296,11 +386,11 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
                                  metadata={'image_size': expected_image_size})
         fake_volumes = {fake_volume_uuid: fake_volume}
 
-        with mock.patch.object(cinder, 'get_cinderclient') as mocked_cc:
+        with mock.patch.object(cinder.Store, 'get_cinderclient') as mocked_cc:
             mocked_cc.return_value = FakeObject(client=fake_client,
                                                 volumes=fake_volumes)
 
-            uri = 'cinder://%s' % fake_volume_uuid
+            uri = 'cinder://cinder1/%s' % fake_volume_uuid
             loc = location.get_location_from_uri_and_backend(uri,
                                                              "cinder1",
                                                              conf=self.conf)
@@ -314,7 +404,7 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
         expected_file_contents = b"*" * expected_size
         image_file = six.BytesIO(expected_file_contents)
         expected_checksum = hashlib.md5(expected_file_contents).hexdigest()
-        expected_location = 'cinder://%s' % fake_volume.id
+        expected_location = 'cinder://%s/%s' % (backend, fake_volume.id)
         fake_client = FakeObject(auth_token=None, management_url=None)
         fake_volume.manager.get.return_value = fake_volume
         fake_volumes = FakeObject(create=mock.Mock(return_value=fake_volume))
@@ -325,7 +415,7 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
             self.assertEqual('wb', mode)
             yield volume_file
 
-        with mock.patch.object(cinder, 'get_cinderclient') as mock_cc, \
+        with mock.patch.object(cinder.Store, 'get_cinderclient') as mock_cc, \
                 mock.patch.object(self.store, '_open_cinder_volume',
                                   side_effect=fake_open):
             mock_cc.return_value = FakeObject(client=fake_client,
@@ -345,7 +435,7 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
                           'glance_image_id': expected_image_id,
                           'image_size': str(expected_size)},
                 volume_type='some_type')
-            self.assertEqual(backend, metadata["backend"])
+            self.assertEqual(backend, metadata["store"])
 
     def test_cinder_add(self):
         fake_volume = mock.MagicMock(id=str(uuid.uuid4()),
@@ -381,11 +471,11 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
         fake_volume = FakeObject(delete=mock.Mock())
         fake_volumes = {fake_volume_uuid: fake_volume}
 
-        with mock.patch.object(cinder, 'get_cinderclient') as mocked_cc:
+        with mock.patch.object(cinder.Store, 'get_cinderclient') as mocked_cc:
             mocked_cc.return_value = FakeObject(client=fake_client,
                                                 volumes=fake_volumes)
 
-            uri = 'cinder://%s' % fake_volume_uuid
+            uri = 'cinder://cinder1/%s' % fake_volume_uuid
             loc = location.get_location_from_uri_and_backend(uri,
                                                              "cinder1",
                                                              conf=self.conf)
