@@ -13,7 +13,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
+from unittest import mock
+
 from oslo_config import cfg
 from oslo_utils import units
 import six
@@ -35,6 +36,9 @@ class MockRados(object):
     class Error(Exception):
         pass
 
+    class ObjectNotFound(Exception):
+        pass
+
     class ioctx(object):
         def __init__(self, *args, **kwargs):
             pass
@@ -89,6 +93,9 @@ class MockRBD(object):
     class InvalidArgument(Exception):
         pass
 
+    class NoSpace(Exception):
+        pass
+
     class Image(object):
 
         def __init__(self, *args, **kwargs):
@@ -187,7 +194,9 @@ class TestMultiStore(base.MultiStoreBaseTest,
         # Ensure stores + locations cleared
         g_location.SCHEME_TO_CLS_BACKEND_MAP = {}
 
-        store.create_multi_stores(self.conf)
+        with mock.patch.object(rbd_store.Store, '_set_url_prefix'):
+            store.create_multi_stores(self.conf)
+
         self.addCleanup(setattr, g_location, 'SCHEME_TO_CLS_BACKEND_MAP',
                         dict())
         self.addCleanup(self.conf.reset)
@@ -209,6 +218,10 @@ class TestMultiStore(base.MultiStoreBaseTest,
         self.data_len = 3 * units.Ki
         self.data_iter = six.BytesIO(b'*' * self.data_len)
 
+    def test_location_url_prefix_is_set(self):
+        expected_url_prefix = "rbd://"
+        self.assertEqual(expected_url_prefix, self.store.url_prefix)
+
     def test_add_w_image_size_zero(self):
         """Assert that correct size is returned even though 0 was provided."""
         self.store.chunk_size = units.Ki
@@ -219,7 +232,7 @@ class TestMultiStore(base.MultiStoreBaseTest,
                 self.assertTrue(resize.called)
                 self.assertTrue(write.called)
                 self.assertEqual(ret[1], self.data_len)
-                self.assertEqual("ceph1", ret[3]['backend'])
+                self.assertEqual("ceph1", ret[3]['store'])
 
     def test_add_w_image_size_zero_to_different_backend(self):
         """Assert that correct size is returned even though 0 was provided."""
@@ -243,7 +256,7 @@ class TestMultiStore(base.MultiStoreBaseTest,
                 self.assertTrue(resize.called)
                 self.assertTrue(write.called)
                 self.assertEqual(ret[1], self.data_len)
-                self.assertEqual("ceph2", ret[3]['backend'])
+                self.assertEqual("ceph2", ret[3]['store'])
 
     @mock.patch.object(MockRBD.Image, '__enter__')
     @mock.patch.object(rbd_store.Store, '_create_image')
@@ -433,11 +446,12 @@ class TestMultiStore(base.MultiStoreBaseTest,
     @mock.patch.object(MockRados.Rados, 'connect', side_effect=MockRados.Error)
     def test_rados_connect_error(self, _):
         rbd_store.rados.Error = MockRados.Error
+        rbd_store.rados.ObjectNotFound = MockRados.ObjectNotFound
 
         def test():
             with self.store.get_connection('conffile', 'rados_id'):
                 pass
-        self.assertRaises(exceptions.BackendException, test)
+        self.assertRaises(exceptions.BadStoreConfiguration, test)
 
     def test_create_image_conf_features(self):
         # Tests that we use non-0 features from ceph.conf and cast to int.
