@@ -16,9 +16,9 @@
 """Tests the Multiple VMware Datastore backend store"""
 
 import hashlib
+from unittest import mock
 import uuid
 
-import mock
 from oslo_config import cfg
 from oslo_utils import units
 from oslo_vmware import api
@@ -115,8 +115,8 @@ class TestMultiStore(base.MultiStoreBaseTest,
                     vmware_store_image_dir='/openstack_glance_1')
         # Ensure stores + locations cleared
         location.SCHEME_TO_CLS_BACKEND_MAP = {}
-
         store.create_multi_stores(self.conf)
+
         self.addCleanup(setattr, location, 'SCHEME_TO_CLS_BACKEND_MAP',
                         dict())
         self.addCleanup(self.conf.reset)
@@ -131,6 +131,10 @@ class TestMultiStore(base.MultiStoreBaseTest,
     def _mock_http_connection(self):
         return mock.patch('six.moves.http_client.HTTPConnection')
 
+    def test_location_url_prefix_is_set(self):
+        expected_url_prefix = "vsphere://127.0.0.1/openstack_glance"
+        self.assertEqual(expected_url_prefix, self.store.url_prefix)
+
     @mock.patch('oslo_vmware.api.VMwareAPISession')
     def test_get(self, mock_api_session):
         """Test a "normal" retrieval of an image in chunks."""
@@ -192,7 +196,7 @@ class TestMultiStore(base.MultiStoreBaseTest,
                     expected_image_id, image, expected_size)
                 _, kwargs = HttpConn.call_args
                 self.assertEqual(expected_headers, kwargs['headers'])
-                self.assertEqual("vmware1", metadata["backend"])
+                self.assertEqual("vmware1", metadata["store"])
 
         self.assertEqual(utils.sort_url_by_qs_keys(expected_location),
                          utils.sort_url_by_qs_keys(location))
@@ -227,7 +231,7 @@ class TestMultiStore(base.MultiStoreBaseTest,
                 HttpConn.return_value = utils.fake_response()
                 location, size, checksum, metadata = self.store.add(
                     expected_image_id, image, 0)
-                self.assertEqual("vmware1", metadata["backend"])
+                self.assertEqual("vmware1", metadata["store"])
 
         self.assertEqual(utils.sort_url_by_qs_keys(expected_location),
                          utils.sort_url_by_qs_keys(location))
@@ -247,7 +251,7 @@ class TestMultiStore(base.MultiStoreBaseTest,
             HttpConn.return_value = utils.fake_response()
             location, size, checksum, multihash, metadata = self.store.add(
                 image_id, image, size, self.hash_algo, verifier=verifier)
-            self.assertEqual("vmware1", metadata["backend"])
+            self.assertEqual("vmware1", metadata["store"])
 
         fake_reader.assert_called_with(image, self.hash_algo, verifier)
 
@@ -264,7 +268,7 @@ class TestMultiStore(base.MultiStoreBaseTest,
             HttpConn.return_value = utils.fake_response()
             location, size, checksum, multihash, metadata = self.store.add(
                 image_id, image, 0, self.hash_algo, verifier=verifier)
-            self.assertEqual("vmware1", metadata["backend"])
+            self.assertEqual("vmware1", metadata["store"])
 
         fake_reader.assert_called_with(image, self.hash_algo, verifier)
 
@@ -391,7 +395,7 @@ class TestMultiStore(base.MultiStoreBaseTest,
         parts = self.store._parse_datastore_info_and_weight(datastore)
         self.assertEqual('a', parts[0])
         self.assertEqual('b', parts[1])
-        self.assertEqual('100', parts[2])
+        self.assertEqual(100, parts[2])
 
     def test_parse_datastore_info_and_weight_default_weight(self):
         datastore = 'a:b'
