@@ -13,23 +13,15 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import pkg_resources
+import stevedore
 from testtools import matchers
 
 from glance_store import backend
 from glance_store.tests import base
 
 
-def load_entry_point(entry_point, verify_requirements=False):
-    """Load an entry-point without requiring dependencies."""
-    resolve = getattr(entry_point, 'resolve', None)
-    require = getattr(entry_point, 'require', None)
-    if resolve is not None and require is not None:
-        if verify_requirements:
-            entry_point.require()
-        return entry_point.resolve()
-    else:
-        return entry_point.load(require=verify_requirements)
+def on_load_failure_callback(*args, **kwargs):
+    raise
 
 
 class OptsTestCase(base.StoreBaseTest):
@@ -53,11 +45,16 @@ class OptsTestCase(base.StoreBaseTest):
     def _test_entry_point(self, namespace,
                           expected_opt_groups, expected_opt_names):
         opt_list = None
-        for ep in pkg_resources.iter_entry_points('oslo.config.opts'):
-            if ep.name == namespace:
-                list_fn = load_entry_point(ep)
-                opt_list = list_fn()
-                break
+        mgr = stevedore.NamedExtensionManager(
+            'oslo.config.opts',
+            names=[namespace],
+            invoke_on_load=False,
+            on_load_failure_callback=on_load_failure_callback,
+        )
+        for ext in mgr:
+            list_fn = ext.plugin
+            opt_list = list_fn()
+            break
 
         self.assertIsNotNone(opt_list)
 
@@ -70,12 +67,12 @@ class OptsTestCase(base.StoreBaseTest):
         expected_opt_names = [
             'default_store',
             'stores',
-            'store_capabilities_update_min_interval',
             'cinder_api_insecure',
             'cinder_ca_certificates_file',
             'cinder_catalog_info',
             'cinder_endpoint_template',
             'cinder_http_retries',
+            'cinder_mount_point_base',
             'cinder_os_region_name',
             'cinder_state_transition_timeout',
             'cinder_store_auth_address',
@@ -83,6 +80,8 @@ class OptsTestCase(base.StoreBaseTest):
             'cinder_store_password',
             'cinder_store_project_name',
             'cinder_volume_type',
+            'cinder_use_multipath',
+            'cinder_enforce_multipath',
             'default_swift_reference',
             'https_insecure',
             'filesystem_store_chunk_size',
@@ -90,18 +89,26 @@ class OptsTestCase(base.StoreBaseTest):
             'filesystem_store_datadirs',
             'filesystem_store_file_perm',
             'filesystem_store_metadata_file',
+            'filesystem_thin_provisioning',
             'http_proxy_information',
             'https_ca_certificates_file',
             'rbd_store_ceph_conf',
             'rbd_store_chunk_size',
             'rbd_store_pool',
             'rbd_store_user',
+            'rbd_thin_provisioning',
             'rados_connect_timeout',
             'rootwrap_config',
+            's3_store_access_key',
+            's3_store_bucket',
+            's3_store_bucket_url_format',
+            's3_store_create_bucket_on_put',
+            's3_store_host',
+            's3_store_secret_key',
+            's3_store_large_object_size',
+            's3_store_large_object_chunk_size',
+            's3_store_thread_pools',
             'swift_store_expire_soon_interval',
-            'sheepdog_store_address',
-            'sheepdog_store_chunk_size',
-            'sheepdog_store_port',
             'swift_store_admin_tenants',
             'swift_store_auth_address',
             'swift_store_cacert',
