@@ -16,16 +16,16 @@
 """Tests the Swift backend store"""
 
 import copy
+from unittest import mock
+
 import fixtures
 import hashlib
-import mock
 import tempfile
 import uuid
 
 from oslo_config import cfg
 from oslo_utils import encodeutils
 from oslo_utils import units
-from oslotest import moxstubout
 import requests_mock
 import six
 from six import moves
@@ -47,8 +47,8 @@ from glance_store.tests.unit import test_store_capabilities
 
 CONF = cfg.CONF
 
-FAKE_UUID = lambda: str(uuid.uuid4())
-FAKE_UUID2 = lambda: str(uuid.uuid4())
+FAKE_UUID = lambda: str(uuid.uuid4())  # noqa: E731
+FAKE_UUID2 = lambda: str(uuid.uuid4())  # noqa: E731
 
 Store = swift.Store
 FIVE_KB = 5 * units.Ki
@@ -64,174 +64,6 @@ SWIFT_CONF = {'swift_store_auth_address': 'localhost:8080',
               }
 
 
-# We stub out as little as possible to ensure that the code paths
-# between swift and swiftclient are tested
-# thoroughly
-def stub_out_swiftclient(stubs, swift_store_auth_version):
-    fixture_containers = ['glance']
-    fixture_container_headers = {}
-    fixture_headers = {
-        'glance/%s' % FAKE_UUID: {
-            'content-length': FIVE_KB,
-            'etag': 'c2e5db72bd7fd153f53ede5da5a06de3'
-        },
-        'glance/%s' % FAKE_UUID2: {'x-static-large-object': 'true', },
-    }
-    fixture_objects = {'glance/%s' % FAKE_UUID: six.BytesIO(b"*" * FIVE_KB),
-                       'glance/%s' % FAKE_UUID2: six.BytesIO(b"*" * FIVE_KB), }
-
-    def fake_head_container(url, token, container, **kwargs):
-        if container not in fixture_containers:
-            msg = "No container %s found" % container
-            status = http_client.NOT_FOUND
-            raise swiftclient.ClientException(msg, http_status=status)
-        return fixture_container_headers
-
-    def fake_put_container(url, token, container, **kwargs):
-        fixture_containers.append(container)
-
-    def fake_post_container(url, token, container, headers, **kwargs):
-        for key, value in headers.items():
-            fixture_container_headers[key] = value
-
-    def fake_put_object(url, token, container, name, contents, **kwargs):
-        # PUT returns the ETag header for the newly-added object
-        # Large object manifest...
-        global SWIFT_PUT_OBJECT_CALLS
-        SWIFT_PUT_OBJECT_CALLS += 1
-        CHUNKSIZE = 64 * units.Ki
-        fixture_key = "%s/%s" % (container, name)
-        if fixture_key not in fixture_headers:
-            if kwargs.get('headers'):
-                etag = kwargs['headers']['ETag']
-                manifest = kwargs.get('headers').get('X-Object-Manifest')
-                fixture_headers[fixture_key] = {'manifest': True,
-                                                'etag': etag,
-                                                'x-object-manifest': manifest}
-                fixture_objects[fixture_key] = None
-                return etag
-            if hasattr(contents, 'read'):
-                fixture_object = six.BytesIO()
-                read_len = 0
-                chunk = contents.read(CHUNKSIZE)
-                checksum = hashlib.md5()
-                while chunk:
-                    fixture_object.write(chunk)
-                    read_len += len(chunk)
-                    checksum.update(chunk)
-                    chunk = contents.read(CHUNKSIZE)
-                etag = checksum.hexdigest()
-            else:
-                fixture_object = six.BytesIO(contents)
-                read_len = len(contents)
-                etag = hashlib.md5(fixture_object.getvalue()).hexdigest()
-            if read_len > MAX_SWIFT_OBJECT_SIZE:
-                msg = ('Image size:%d exceeds Swift max:%d' %
-                       (read_len, MAX_SWIFT_OBJECT_SIZE))
-                raise swiftclient.ClientException(
-                    msg, http_status=http_client.REQUEST_ENTITY_TOO_LARGE)
-            fixture_objects[fixture_key] = fixture_object
-            fixture_headers[fixture_key] = {
-                'content-length': read_len,
-                'etag': etag}
-            return etag
-        else:
-            msg = ("Object PUT failed - Object with key %s already exists"
-                   % fixture_key)
-            raise swiftclient.ClientException(msg,
-                                              http_status=http_client.CONFLICT)
-
-    def fake_get_object(conn, container, name, **kwargs):
-        # GET returns the tuple (list of headers, file object)
-        fixture_key = "%s/%s" % (container, name)
-        if fixture_key not in fixture_headers:
-            msg = "Object GET failed"
-            status = http_client.NOT_FOUND
-            raise swiftclient.ClientException(msg, http_status=status)
-
-        byte_range = None
-        headers = kwargs.get('headers', dict())
-        if headers is not None:
-            headers = dict((k.lower(), v) for k, v in headers.items())
-            if 'range' in headers:
-                byte_range = headers.get('range')
-
-        fixture = fixture_headers[fixture_key]
-        if 'manifest' in fixture:
-            # Large object manifest... we return a file containing
-            # all objects with prefix of this fixture key
-            chunk_keys = sorted([k for k in fixture_headers.keys()
-                                 if k.startswith(fixture_key) and
-                                 k != fixture_key])
-            result = six.BytesIO()
-            for key in chunk_keys:
-                result.write(fixture_objects[key].getvalue())
-        else:
-            result = fixture_objects[fixture_key]
-
-        if byte_range is not None:
-            start = int(byte_range.split('=')[1].strip('-'))
-            result = six.BytesIO(result.getvalue()[start:])
-            fixture_headers[fixture_key]['content-length'] = len(
-                result.getvalue())
-
-        return fixture_headers[fixture_key], result
-
-    def fake_head_object(url, token, container, name, **kwargs):
-        # HEAD returns the list of headers for an object
-        try:
-            fixture_key = "%s/%s" % (container, name)
-            return fixture_headers[fixture_key]
-        except KeyError:
-            msg = "Object HEAD failed - Object does not exist"
-            status = http_client.NOT_FOUND
-            raise swiftclient.ClientException(msg, http_status=status)
-
-    def fake_delete_object(url, token, container, name, **kwargs):
-        # DELETE returns nothing
-        fixture_key = "%s/%s" % (container, name)
-        if fixture_key not in fixture_headers:
-            msg = "Object DELETE failed - Object does not exist"
-            status = http_client.NOT_FOUND
-            raise swiftclient.ClientException(msg, http_status=status)
-        else:
-            del fixture_headers[fixture_key]
-            del fixture_objects[fixture_key]
-
-    def fake_http_connection(*args, **kwargs):
-        return None
-
-    def fake_get_auth(url, user, key, auth_version, **kwargs):
-        if url is None:
-            return None, None
-        if 'http' in url and '://' not in url:
-            raise ValueError('Invalid url %s' % url)
-        # Check the auth version against the configured value
-        if swift_store_auth_version != auth_version:
-            msg = 'AUTHENTICATION failed (version mismatch)'
-            raise swiftclient.ClientException(msg)
-        return None, None
-
-    stubs.Set(swiftclient.client,
-              'head_container', fake_head_container)
-    stubs.Set(swiftclient.client,
-              'put_container', fake_put_container)
-    stubs.Set(swiftclient.client,
-              'post_container', fake_post_container)
-    stubs.Set(swiftclient.client,
-              'put_object', fake_put_object)
-    stubs.Set(swiftclient.client,
-              'delete_object', fake_delete_object)
-    stubs.Set(swiftclient.client,
-              'head_object', fake_head_object)
-    stubs.Set(swiftclient.client.Connection,
-              'get_object', fake_get_object)
-    stubs.Set(swiftclient.client,
-              'get_auth', fake_get_auth)
-    stubs.Set(swiftclient.client,
-              'http_connection', fake_http_connection)
-
-
 class SwiftTests(object):
 
     def mock_keystone_client(self):
@@ -240,6 +72,175 @@ class SwiftTests(object):
         swift.ks_session = mock.MagicMock()
         swift.ks_client = mock.MagicMock()
 
+    def stub_out_swiftclient(self, swift_store_auth_version):
+        fixture_containers = ['glance']
+        fixture_container_headers = {}
+        fixture_headers = {
+            'glance/%s' % FAKE_UUID: {
+                'content-length': FIVE_KB,
+                'etag': 'c2e5db72bd7fd153f53ede5da5a06de3'
+            },
+            'glance/%s' % FAKE_UUID2: {'x-static-large-object': 'true', },
+        }
+        fixture_objects = {
+            'glance/%s' % FAKE_UUID: six.BytesIO(b"*" * FIVE_KB),
+            'glance/%s' % FAKE_UUID2: six.BytesIO(b"*" * FIVE_KB),
+        }
+
+        def fake_head_container(url, token, container, **kwargs):
+            if container not in fixture_containers:
+                msg = "No container %s found" % container
+                status = http_client.NOT_FOUND
+                raise swiftclient.ClientException(msg, http_status=status)
+            return fixture_container_headers
+
+        def fake_put_container(url, token, container, **kwargs):
+            fixture_containers.append(container)
+
+        def fake_post_container(url, token, container, headers, **kwargs):
+            for key, value in headers.items():
+                fixture_container_headers[key] = value
+
+        def fake_put_object(url, token, container, name, contents, **kwargs):
+            # PUT returns the ETag header for the newly-added object
+            # Large object manifest...
+            global SWIFT_PUT_OBJECT_CALLS
+            SWIFT_PUT_OBJECT_CALLS += 1
+            CHUNKSIZE = 64 * units.Ki
+            fixture_key = "%s/%s" % (container, name)
+            if fixture_key not in fixture_headers:
+                if kwargs.get('headers'):
+                    manifest = kwargs.get('headers').get('X-Object-Manifest')
+                    etag = kwargs.get('headers') \
+                                 .get('ETag', hashlib.md5(b'').hexdigest())
+                    fixture_headers[fixture_key] = {
+                        'manifest': True,
+                        'etag': etag,
+                        'x-object-manifest': manifest
+                    }
+                    fixture_objects[fixture_key] = None
+                    return etag
+                if hasattr(contents, 'read'):
+                    fixture_object = six.BytesIO()
+                    read_len = 0
+                    chunk = contents.read(CHUNKSIZE)
+                    checksum = hashlib.md5()
+                    while chunk:
+                        fixture_object.write(chunk)
+                        read_len += len(chunk)
+                        checksum.update(chunk)
+                        chunk = contents.read(CHUNKSIZE)
+                    etag = checksum.hexdigest()
+                else:
+                    fixture_object = six.BytesIO(contents)
+                    read_len = len(contents)
+                    etag = hashlib.md5(fixture_object.getvalue()).hexdigest()
+                if read_len > MAX_SWIFT_OBJECT_SIZE:
+                    msg = ('Image size:%d exceeds Swift max:%d' %
+                           (read_len, MAX_SWIFT_OBJECT_SIZE))
+                    raise swiftclient.ClientException(
+                        msg, http_status=http_client.REQUEST_ENTITY_TOO_LARGE)
+                fixture_objects[fixture_key] = fixture_object
+                fixture_headers[fixture_key] = {
+                    'content-length': read_len,
+                    'etag': etag}
+                return etag
+            else:
+                msg = ("Object PUT failed - Object with key %s already exists"
+                       % fixture_key)
+                raise swiftclient.ClientException(
+                    msg, http_status=http_client.CONFLICT)
+
+        def fake_get_object(conn, container, name, **kwargs):
+            # GET returns the tuple (list of headers, file object)
+            fixture_key = "%s/%s" % (container, name)
+            if fixture_key not in fixture_headers:
+                msg = "Object GET failed"
+                status = http_client.NOT_FOUND
+                raise swiftclient.ClientException(msg, http_status=status)
+
+            byte_range = None
+            headers = kwargs.get('headers', dict())
+            if headers is not None:
+                headers = dict((k.lower(), v) for k, v in headers.items())
+                if 'range' in headers:
+                    byte_range = headers.get('range')
+
+            fixture = fixture_headers[fixture_key]
+            if 'manifest' in fixture:
+                # Large object manifest... we return a file containing
+                # all objects with prefix of this fixture key
+                chunk_keys = sorted([k for k in fixture_headers.keys()
+                                     if k.startswith(fixture_key) and
+                                     k != fixture_key])
+                result = six.BytesIO()
+                for key in chunk_keys:
+                    result.write(fixture_objects[key].getvalue())
+            else:
+                result = fixture_objects[fixture_key]
+
+            if byte_range is not None:
+                start = int(byte_range.split('=')[1].strip('-'))
+                result = six.BytesIO(result.getvalue()[start:])
+                fixture_headers[fixture_key]['content-length'] = len(
+                    result.getvalue())
+
+            return fixture_headers[fixture_key], result
+
+        def fake_head_object(url, token, container, name, **kwargs):
+            # HEAD returns the list of headers for an object
+            try:
+                fixture_key = "%s/%s" % (container, name)
+                return fixture_headers[fixture_key]
+            except KeyError:
+                msg = "Object HEAD failed - Object does not exist"
+                status = http_client.NOT_FOUND
+                raise swiftclient.ClientException(msg, http_status=status)
+
+        def fake_delete_object(url, token, container, name, **kwargs):
+            # DELETE returns nothing
+            fixture_key = "%s/%s" % (container, name)
+            if fixture_key not in fixture_headers:
+                msg = "Object DELETE failed - Object does not exist"
+                status = http_client.NOT_FOUND
+                raise swiftclient.ClientException(msg, http_status=status)
+            else:
+                del fixture_headers[fixture_key]
+                del fixture_objects[fixture_key]
+
+        def fake_http_connection(*args, **kwargs):
+            return None
+
+        def fake_get_auth(url, user, key, auth_version, **kwargs):
+            if url is None:
+                return None, None
+            if 'http' in url and '://' not in url:
+                raise ValueError('Invalid url %s' % url)
+            # Check the auth version against the configured value
+            if swift_store_auth_version != auth_version:
+                msg = 'AUTHENTICATION failed (version mismatch)'
+                raise swiftclient.ClientException(msg)
+            return None, None
+
+        self.useFixture(fixtures.MockPatch(
+            'swiftclient.client.head_container', fake_head_container))
+        self.useFixture(fixtures.MockPatch(
+            'swiftclient.client.put_container', fake_put_container))
+        self.useFixture(fixtures.MockPatch(
+            'swiftclient.client.post_container', fake_post_container))
+        self.useFixture(fixtures.MockPatch(
+            'swiftclient.client.put_object', fake_put_object))
+        self.useFixture(fixtures.MockPatch(
+            'swiftclient.client.delete_object', fake_delete_object))
+        self.useFixture(fixtures.MockPatch(
+            'swiftclient.client.head_object', fake_head_object))
+        self.useFixture(fixtures.MockPatch(
+            'swiftclient.client.Connection.get_object', fake_get_object))
+        self.useFixture(fixtures.MockPatch(
+            'swiftclient.client.get_auth', fake_get_auth))
+        self.useFixture(fixtures.MockPatch(
+            'swiftclient.client.http_connection', fake_http_connection))
+
     @property
     def swift_store_user(self):
         return 'tenant:user1'
@@ -407,7 +408,7 @@ class SwiftTests(object):
         loc, size, checksum, metadata = self.store.add(
             expected_image_id, image_swift, expected_swift_size)
 
-        self.assertEqual("swift1", metadata["backend"])
+        self.assertEqual("swift1", metadata["store"])
         self.assertEqual(expected_location, loc)
         self.assertEqual(expected_swift_size, size)
         self.assertEqual(expected_checksum, checksum)
@@ -446,7 +447,7 @@ class SwiftTests(object):
         location, size, checksum, arg = self.store.add(expected_image_id,
                                                        image_swift,
                                                        expected_swift_size)
-        self.assertEqual("swift1", arg['backend'])
+        self.assertEqual("swift1", arg['store'])
         self.assertEqual(expected_location, location)
 
     @mock.patch('glance_store._drivers.swift.utils'
@@ -492,7 +493,7 @@ class SwiftTests(object):
                                                   expected_swift_size,
                                                   context=ctxt)
 
-        self.assertEqual("swift1", metadata['backend'])
+        self.assertEqual("swift1", metadata['store'])
         # ensure that image add uses user's context
         self.assertEqual(expected_location, loc)
 
@@ -535,7 +536,7 @@ class SwiftTests(object):
                                                            image_swift,
                                                            expected_swift_size)
 
-            self.assertEqual("swift1", metadata['backend'])
+            self.assertEqual("swift1", metadata['store'])
             self.assertEqual(expected_location, loc)
             self.assertEqual(expected_swift_size, size)
             self.assertEqual(expected_checksum, checksum)
@@ -615,7 +616,7 @@ class SwiftTests(object):
                                                        image_swift,
                                                        expected_swift_size)
 
-        self.assertEqual("swift1", metadata['backend'])
+        self.assertEqual("swift1", metadata['store'])
         self.assertEqual(expected_location, loc)
         self.assertEqual(expected_swift_size, size)
         self.assertEqual(expected_checksum, checksum)
@@ -664,7 +665,7 @@ class SwiftTests(object):
                                                        image_swift,
                                                        expected_swift_size)
 
-        self.assertEqual("swift1", metadata['backend'])
+        self.assertEqual("swift1", metadata['store'])
         self.assertEqual(expected_location, loc)
         self.assertEqual(expected_swift_size, size)
         self.assertEqual(expected_checksum, checksum)
@@ -849,7 +850,7 @@ class SwiftTests(object):
                                                        expected_swift_size,
                                                        context=ctxt)
 
-        self.assertEqual("swift1", metadata['backend'])
+        self.assertEqual("swift1", metadata['store'])
         self.assertEqual(expected_location, location)
 
     @mock.patch('glance_store._drivers.swift.utils'
@@ -887,7 +888,7 @@ class SwiftTests(object):
             self.store.large_object_chunk_size = orig_temp_size
             self.store.large_object_size = orig_max_size
 
-        self.assertEqual("swift1", metadata['backend'])
+        self.assertEqual("swift1", metadata['store'])
         self.assertEqual(expected_location, loc)
         self.assertEqual(expected_swift_size, size)
         self.assertEqual(expected_checksum, checksum)
@@ -948,7 +949,7 @@ class SwiftTests(object):
             self.store.large_object_size = orig_max_size
             MAX_SWIFT_OBJECT_SIZE = orig_max_swift_object_size
 
-        self.assertEqual("swift1", metadata['backend'])
+        self.assertEqual("swift1", metadata['store'])
         self.assertEqual(expected_location, loc)
         self.assertEqual(expected_swift_size, size)
         self.assertEqual(expected_checksum, checksum)
@@ -964,6 +965,12 @@ class SwiftTests(object):
         self.assertEqual(expected_swift_contents, new_image_contents)
         self.assertEqual(expected_swift_size, new_image_swift_size)
 
+    def test_location_url_prefix_is_set(self):
+        self.store = Store(self.conf, backend="swift1")
+        self.store.configure()
+        expected_url_prefix = "swift+config://ref1/glance/"
+        self.assertEqual(expected_url_prefix, self.store.url_prefix)
+
     def test_add_already_existing(self):
         """
         Tests that adding an image with an existing identifier
@@ -1245,17 +1252,26 @@ class SwiftTests(object):
 
     def test_init_client_multi_tenant(self):
         """Test that keystone client was initialized correctly"""
-        self._init_client(verify=True, swift_store_multi_tenant=True,
-                          swift_store_config_file=None)
+        with mock.patch.object(swift.MultiTenantStore, '_set_url_prefix'):
+            self._init_client(verify=True, swift_store_multi_tenant=True,
+                              swift_store_config_file=None)
+
+    def test_init_client_multi_tenant_swift_cacert(self):
+        """Test that keystone client was initialized with swift cacert"""
+        with mock.patch.object(swift.MultiTenantStore, '_set_url_prefix'):
+            self._init_client(verify='/foo/bar', swift_store_multi_tenant=True,
+                              swift_store_config_file=None,
+                              swift_store_cacert='/foo/bar')
 
     def test_init_client_multi_tenant_insecure(self):
         """
         Test that keystone client was initialized correctly with no
         certificate verification.
         """
-        self._init_client(verify=False, swift_store_multi_tenant=True,
-                          swift_store_auth_insecure=True,
-                          swift_store_config_file=None)
+        with mock.patch.object(swift.MultiTenantStore, '_set_url_prefix'):
+            self._init_client(verify=False, swift_store_multi_tenant=True,
+                              swift_store_auth_insecure=True,
+                              swift_store_config_file=None)
 
     @mock.patch("glance_store._drivers.swift.store.ks_identity")
     @mock.patch("glance_store._drivers.swift.store.ks_session")
@@ -1376,9 +1392,7 @@ class TestStoreAuthV1(base.MultiStoreBaseTest, SwiftTests,
         self.swift_config_file = self.copy_data_file(conf_file, self.test_dir)
         config.update({'swift_store_config_file': self.swift_config_file})
 
-        moxfixture = self.useFixture(moxstubout.MoxStubout())
-        self.stubs = moxfixture.stubs
-        stub_out_swiftclient(self.stubs, config['swift_store_auth_version'])
+        self.stub_out_swiftclient(config['swift_store_auth_version'])
         self.mock_keystone_client()
         self.store = Store(self.conf, backend="swift1")
         self.config(group="swift1", **config)
@@ -1446,8 +1460,8 @@ class TestStoreAuthV3(TestStoreAuthV1):
             auth_url=loc.store_location.swift_url + '/',
             username=username, password="key",
             project_name=tenant,
-            project_domain_id='default', project_domain_name=None,
-            user_domain_id='default', user_domain_name=None,)
+            project_domain_id='default', project_domain_name='default',
+            user_domain_id='default', user_domain_name='default',)
         mock_session.Session.assert_called_once_with(
             auth=mock_identity.V3Password(), verify=True)
         mock_client.Client.assert_called_once_with(
@@ -1502,9 +1516,8 @@ class TestSingleTenantStoreConnections(base.MultiStoreBaseTest):
                         dict())
         self.test_dir = self.useFixture(fixtures.TempDir()).path
 
-        moxfixture = self.useFixture(moxstubout.MoxStubout())
-        self.stubs = moxfixture.stubs
-        self.stubs.Set(swiftclient, 'Connection', FakeConnection)
+        self.useFixture(fixtures.MockPatch(
+            'swiftclient.Connection', FakeConnection))
         self.store = swift.SingleTenantStore(self.conf, backend="swift1")
         self.store.configure()
         specs = {'scheme': 'swift',
@@ -1717,9 +1730,8 @@ class TestMultiTenantStoreConnections(base.MultiStoreBaseTest):
                         dict())
         self.test_dir = self.useFixture(fixtures.TempDir()).path
 
-        moxfixture = self.useFixture(moxstubout.MoxStubout())
-        self.stubs = moxfixture.stubs
-        self.stubs.Set(swiftclient, 'Connection', FakeConnection)
+        self.useFixture(fixtures.MockPatch(
+            'swiftclient.Connection', FakeConnection))
         self.context = mock.MagicMock(
             user='tenant:user1', tenant='tenant', auth_token='0123')
         self.store = swift.MultiTenantStore(self.conf, backend="swift1")
@@ -1913,8 +1925,6 @@ class TestCreatingLocations(base.MultiStoreBaseTest):
                         dict())
         self.test_dir = self.useFixture(fixtures.TempDir()).path
 
-        moxfixture = self.useFixture(moxstubout.MoxStubout())
-        self.stubs = moxfixture.stubs
         config = copy.deepcopy(SWIFT_CONF)
         self.store = Store(self.conf, backend="swift1")
         self.config(group="swift1", **config)
