@@ -16,9 +16,9 @@
 """Tests the VMware Datastore backend store"""
 
 import hashlib
+from unittest import mock
 import uuid
 
-import mock
 from oslo_utils import units
 from oslo_vmware import api
 from oslo_vmware import exceptions as vmware_exceptions
@@ -396,7 +396,7 @@ class TestStore(base.StoreBaseTest,
         parts = self.store._parse_datastore_info_and_weight(datastore)
         self.assertEqual('a', parts[0])
         self.assertEqual('b', parts[1])
-        self.assertEqual('100', parts[2])
+        self.assertEqual(100, parts[2])
 
     def test_parse_datastore_info_and_weight_default_weight(self):
         datastore = 'a:b'
